import gi
gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')
from gi.repository import Gtk, Adw, GObject, GLib

from .chat_application import _
from .model_selection import ModelSelectionManager
from .debug_utils import debug_print
from .resource_manager import resource_manager

class ModelSelectorWidget(Gtk.Box):
    """
    Widget reutilizable que implementa una UI de selección de modelo con navegación
    en dos pasos (Proveedores -> Modelos) usando Adw.ViewStack.
    Ahora usa la misma lógica que WideModelSelector pero en formato vertical.
    """

    __gsignals__ = {
        'model-selected': (GObject.SignalFlags.RUN_LAST, None, (str,)),
        'api-key-status-changed': (GObject.SignalFlags.RUN_LAST, None, (str, bool, bool))
    }

    def __init__(self, manager=None, config=None, llm_client=None, show_headers=True):
        """
        Inicializa el selector de modelos.
        
        Args:
            manager: ModelSelectionManager opcional a usar. Si no se proporciona,
                   se creará uno nuevo usando config y llm_client.
            config: Configuración opcional si no se proporciona manager.
            llm_client: Cliente LLM opcional si no se proporciona manager.
            show_headers: Si mostrar los headers en las páginas (por defecto True).
        """
        super().__init__(orientation=Gtk.Orientation.VERTICAL, spacing=0)
        
        # Inicializar manager
        self.manager = manager or ModelSelectionManager(config, llm_client)
        self._selected_provider_key = None
        self.show_headers = show_headers

        # Contenedor principal para apilar paneles
        self.stack = Gtk.Stack()
        self.stack.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT_RIGHT)
        self.stack.set_vexpand(True)

        # --- Lista de Proveedores ---
        provider_page = self._create_provider_page()
        self.stack.add_titled(provider_page, "providers", _("Providers"))

        # --- Lista de Modelos ---
        model_page = self._create_model_page()
        self.stack.add_titled(model_page, "models", _("Models"))

        # Añadir stack al widget
        self.append(self.stack)

        # Conectar señales del manager
        self.manager.connect('api-key-changed', self._on_api_key_changed)

    def _create_provider_page(self):
        """Crea la página de selección de proveedor."""
        page = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
        page.set_vexpand(True)

        # Header con título (solo si show_headers es True)
        if self.show_headers:
            header = Adw.HeaderBar()
            header.set_show_end_title_buttons(False)
            header.add_css_class("flat")
            header.set_title_widget(Gtk.Label(label=_("Select Provider")))
            page.append(header)

        # Lista de proveedores en una sola columna
        scroll = Gtk.ScrolledWindow(hscrollbar_policy=Gtk.PolicyType.NEVER,
                                     vscrollbar_policy=Gtk.PolicyType.AUTOMATIC)
        scroll.set_vexpand(True)
        scroll.set_hexpand(True)
        self.provider_list = Gtk.ListBox(selection_mode=Gtk.SelectionMode.SINGLE)
        self.provider_list.add_css_class('navigation-sidebar')
        self.provider_list.connect("row-activated", self._on_provider_row_activated)
        scroll.set_child(self.provider_list)
        page.append(scroll)
        return page

    def _create_model_page(self):
        """Crea la página de selección de modelo."""
        page = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
        page.set_vexpand(True)

        # Header con botón atrás y título
        header = Adw.HeaderBar()
        header.set_show_end_title_buttons(False)
        header.add_css_class("flat")
        back_button = Gtk.Button(icon_name="go-previous-symbolic")
        back_button.connect("clicked", lambda x: self.stack.set_visible_child_name("providers"))
        header.pack_start(back_button)
        header.set_title_widget(Gtk.Label(label=_("Select Model")))
        page.append(header)

        # Banner para API key (inicialmente oculto)
        self.api_key_banner = Adw.Banner(revealed=False)
        gizmo = self.api_key_banner.get_first_child().get_first_child()
        gizmo.set_css_classes(['card'])
        self.api_button = gizmo.get_last_child()
        self.api_key_banner.connect("button-clicked", self._on_banner_button_clicked)
        page.append(self.api_key_banner)

        # Lista de modelos en una sola columna
        scroll = Gtk.ScrolledWindow(hscrollbar_policy=Gtk.PolicyType.NEVER,
                                     vscrollbar_policy=Gtk.PolicyType.AUTOMATIC)
        scroll.set_vexpand(True)
        scroll.set_hexpand(True)
        self.model_list = Gtk.ListBox(selection_mode=Gtk.SelectionMode.SINGLE)
        self.model_list.add_css_class('navigation-sidebar')
        self.model_list.connect("row-activated", self._on_model_row_activated)
        scroll.set_child(self.model_list)
        page.append(scroll)
        return page

    def load_providers(self):
        """Carga y muestra la lista de proveedores."""
        self._clear_list_box(self.provider_list)
        
        if not self.manager.populate_providers_and_group_models():
            # Error al cargar proveedores
            row = Adw.ActionRow(title=_("No models found"))
            row.set_selectable(False)
            self.provider_list.append(row)
            return

        # Ordenar proveedores por nombre
        providers = sorted(
            self.manager.models_by_provider.keys(),
            key=lambda p: self.manager.get_provider_display_name(p).lower() if p else "local/other"
        )

        for provider_key in providers:
            display_name = self.manager.get_provider_display_name(provider_key)
            
            # Pre-cargar modelos para obtener el conteo
            models = self.manager.get_models_for_provider(provider_key)
            model_count = len(models) if models else 0
            
            # Crear fila con subtítulo mostrando el número de modelos
            row = Adw.ActionRow(title=display_name)
            if model_count > 0:
                row.set_subtitle(f"{model_count} {_('models')}")
            else:
                # Verificar si necesita API key para mostrar mensaje apropiado
                key_status = self.manager.check_api_key_status(provider_key)
                if key_status.get('needs_key', False) and not key_status.get('has_key', False):
                    row.set_subtitle(_("API key required"))
                else:
                    row.set_subtitle(_("No models"))
            
            row.set_activatable(True)
            row.add_suffix(resource_manager.create_icon_widget("go-next-symbolic"))
            row.provider_key = provider_key
            debug_print(f"DEBUG: Created row for provider '{provider_key}' (type: {type(provider_key)}) with {model_count} models")
            self.provider_list.append(row)

    def _clear_list_box(self, list_box):
        """Elimina todas las filas de un Gtk.ListBox."""
        child = list_box.get_first_child()
        while child:
            next_child = child.get_next_sibling()
            list_box.remove(child)
            child = next_child

    def _on_provider_row_activated(self, list_box, row):
        """Manejador cuando se selecciona un proveedor."""
        # Usar hasattr para distinguir entre "no tiene atributo" vs "atributo es None"
        if hasattr(row, 'provider_key'):
            provider_key = row.provider_key
            debug_print(f"DEBUG: Row activated - provider_key: {provider_key} (type: {type(provider_key)})")
            debug_print(f"DEBUG: Calling _populate_model_list for {provider_key}")
            self._populate_model_list(provider_key)
            debug_print(f"DEBUG: Setting stack to 'models' page")
            self.stack.set_visible_child_name("models")
        else:
            debug_print(f"DEBUG: Row does not have provider_key attribute, no action taken")

    def _populate_model_list(self, provider_key):
        """Puebla la lista de modelos para el proveedor seleccionado."""
        debug_print(f"DEBUG: _populate_model_list called with provider_key: {provider_key} (type: {type(provider_key)})")
        self._clear_list_box(self.model_list)
        self._selected_provider_key = provider_key
        
        # DEBUG: Imprimir información del proveedor
        debug_print(f"DEBUG: Proveedor seleccionado: {provider_key}")
        
        # Obtener modelos primero para determinar si necesita API key
        models = self.manager.get_models_for_provider(provider_key)
        debug_print(f"DEBUG: Modelos encontrados para {provider_key}: {len(models) if models else 0}")
        if models:
            debug_print(f"DEBUG: Primeros 3 modelos: {[getattr(m, 'model_id', 'NO_ID') for m in models[:3]]}")
        
        # Actualizar banner de API key
        key_status = self.manager.check_api_key_status(provider_key)
        debug_print(f"DEBUG: Key status para {provider_key}: {key_status}")
        
        # Si no hay modelos y el proveedor no es "local/other", asumir que necesita API key
        needs_api_key = key_status.get('needs_key', False)
        if not models and provider_key not in [None, '', 'local', 'local/other', 'ollama']:
            debug_print(f"DEBUG: No hay modelos para {provider_key}, asumiendo que necesita API key")
            needs_api_key = True
        
        if needs_api_key:
            debug_print(f"DEBUG: Proveedor {provider_key} necesita API key")
            if key_status.get('has_key', False):
                debug_print(f"DEBUG: API key ya configurada para {provider_key}")
                self.api_key_banner.set_title(_("API Key is configured"))
                self.api_key_banner.set_button_label(_("Change Key"))
                self.api_button.remove_css_class("error")
                self.api_button.add_css_class("success")
            else:
                debug_print(f"DEBUG: API key NO configurada para {provider_key}")
                self.api_key_banner.set_title(_("API Key Required"))
                self.api_key_banner.set_button_label(_("Set Key"))
                self.api_button.remove_css_class("success")
                self.api_button.add_css_class("error")
            debug_print(f"DEBUG: Mostrando banner para {provider_key}")
            self.api_key_banner.set_revealed(True)
        else:
            debug_print(f"DEBUG: Proveedor {provider_key} NO necesita API key, ocultando banner")
            self.api_key_banner.set_revealed(False)

        # Emitir señal de cambio de estado de API key
        has_key = key_status.get('has_key', False)
        self.emit('api-key-status-changed', provider_key, needs_api_key, has_key)

        if not models:
            # Si no hay modelos pero el proveedor requiere API key, mostrar el banner
            if needs_api_key and not has_key:
                # El banner ya está configurado arriba, solo añadir mensaje explicativo
                debug_print(f"DEBUG: Mostrando mensaje de API key requerida")
                row = Adw.ActionRow(title=_("No models available"))
                row.set_subtitle(_("Configure an API key to access models from this provider"))
                row.set_selectable(False)
                self.model_list.append(row)
            else:
                debug_print(f"DEBUG: Mostrando mensaje genérico de no modelos")
                row = Adw.ActionRow(title=_("No models found for this provider"))
                row.set_selectable(False)
                self.model_list.append(row)
            return

        current_model_id = None
        if self.manager.llm_client:
            current_model_id = self.manager.llm_client.get_model_id()
        if not current_model_id:
            current_model_id = self.manager.config.get('model')

        active_row = None
        for model_obj in models:
            model_id = getattr(model_obj, 'model_id', None)
            model_name = getattr(model_obj, 'name', None) or model_id
            if model_id:
                row = Adw.ActionRow(title=model_name)
                row.set_activatable(True)
                row.model_id = model_id
                self.model_list.append(row)
                if model_id == current_model_id:
                    active_row = row

        if active_row:
            self.model_list.select_row(active_row)

    def _on_model_row_activated(self, list_box, row):
        """Manejador cuando se selecciona un modelo."""
        model_id = getattr(row, 'model_id', None)
        if model_id:
            success = True
            if self.manager.llm_client:
                success = self.manager.llm_client.set_model(model_id)
            if success:
                self.manager.config['model'] = model_id
                self.emit('model-selected', model_id)
                debug_print(f"ModelSelector: Model '{model_id}' selected.")

    def _on_banner_button_clicked(self, banner):
        """Manejador para el clic del botón en el banner de API key."""
        provider_key = getattr(self, '_selected_provider_key', None)
        if not provider_key:
            return

        dialog = Adw.MessageDialog(
            transient_for=self.get_root(),
            modal=True,
            heading=_("Enter API Key"),
            body=f"{_('Enter the API key for')} {self.manager.get_provider_display_name(provider_key)}:",
        )
        dialog.add_response("cancel", _("Cancel"))
        dialog.add_response("set", _("Set Key"))
        dialog.set_response_appearance("set", Adw.ResponseAppearance.SUGGESTED)
        dialog.set_default_response("set")

        key_entry = Gtk.Entry(
            hexpand=True,
            placeholder_text=_("Paste your API key here")
        )
        key_entry.connect("activate", lambda entry: dialog.response("set"))

        dialog.set_extra_child(key_entry)
        dialog.connect("response", self._on_api_key_dialog_response, provider_key, key_entry)
        dialog.present()

    def _on_api_key_dialog_response(self, dialog, response_id, provider_key, key_entry):
        """Manejador de la respuesta del diálogo de API key."""
        if response_id == "set":
            api_key = key_entry.get_text()
            if api_key:
                self.manager.set_api_key(provider_key, api_key)
        dialog.destroy()

    def _on_api_key_changed(self, manager, provider_key):
        """Manejador cuando cambia una API key."""
        debug_print(f"ModelSelector: API key changed for provider '{provider_key}', updating UI.")
        
        # Save current selection to restore after reload
        current_provider = self._selected_provider_key
        
        # Reload dynamic models for new API key
        try:
            self.manager.reload_dynamic_models_only()
            debug_print(f"ModelSelector: Dynamic models reloaded after API key change for '{provider_key}'")
        except Exception as e:
            debug_print(f"ModelSelector: Error reloading dynamic models: {e}")
        
        # Update UI if this provider is currently selected
        if current_provider == provider_key:
            self._populate_model_list(provider_key)
        
        # Update API key status
        key_status = manager.check_api_key_status(provider_key)
        if key_status.get('has_key', False):
            self.api_key_banner.set_title(_("API Key is configured"))
            self.api_key_banner.set_button_label(_("Change Key"))
            self.api_button.remove_css_class("error")
            self.api_button.add_css_class("success")
        else:
            self.api_key_banner.set_title(_("API Key Required"))
            self.api_key_banner.set_button_label(_("Set Key"))
            self.api_button.remove_css_class("success")
            self.api_button.add_css_class("error")

if __name__ == '__main__':
    import sys
    from .model_selection import ModelSelectionManager
    
    class TestApp(Adw.Application):
        def __init__(self):
            super().__init__(application_id="org.example.ModelSelectorTest")
            
        def do_activate(self):
            # Crear ventana principal
            window = Adw.ApplicationWindow(application=self)
            window.set_title(_("Model Selector Test"))
            window.set_default_size(600, 500)
            
            # Crear configuración mock
            mock_config = {}
            
            # Crear el widget selector
            selector = ModelSelectorWidget(config=mock_config)
            
            # Cargar proveedores
            selector.load_providers()
            
            # Conectar señal de modelo seleccionado
            def on_model_selected(widget, model_id):
                debug_print(f"Modelo seleccionado: {model_id}")
            
            selector.connect('model-selected', on_model_selected)
            
            # Agregar a la ventana
            window.set_content(selector)
            window.present()
    
    app = TestApp()
    app.run(sys.argv)



import gi
gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')
from gi.repository import Gtk, Adw, GObject, GLib

from .chat_application import _
from .model_selection import ModelSelectionManager
from .debug_utils import debug_print
from .resource_manager import resource_manager

NO_SELECTION_KEY = "_internal_no_selection_"

class WideModelSelector(Gtk.Box):
    """
    Un widget de selección de modelo compacto usando Gtk.StackSidebar.
    Maneja la selección de proveedor, modelo y la configuración de API keys.
    No tiene títulos ni banners propios, diseñado para ser integrado.
    La gestión del botón de API key y el diálogo se delega al contenedor.
    """
    __gsignals__ = {
        'model-selected': (GObject.SignalFlags.RUN_LAST, None, (str,)),
        # Señal: (provider_key, needs_key, has_key)
        'api-key-status-changed': (GObject.SignalFlags.RUN_LAST, None, (str, bool, bool))
    }

    def __init__(self, manager: ModelSelectionManager = None, config=None, llm_client=None, **kwargs):
        super().__init__(orientation=Gtk.Orientation.HORIZONTAL, spacing=6, **kwargs)

        self.manager = manager or ModelSelectionManager(config, llm_client)
        self._current_provider_key_for_api_dialog = None

        # Stack principal para el contenido (listas de modelos y tarjetas de API key)
        self.content_stack = Gtk.Stack()
        self.content_stack.set_transition_type(Gtk.StackTransitionType.SLIDE_LEFT_RIGHT)
        self.content_stack.set_vexpand(True)
        self.content_stack.set_hexpand(True)

        # Sidebar para navegar el stack de contenido (proveedores)
        self.provider_sidebar = Gtk.StackSidebar()
        self.provider_sidebar.set_stack(self.content_stack)
        self.provider_sidebar.set_vexpand(True)
        listbox = self.provider_sidebar.get_last_child().get_child().get_child()
        listbox.set_valign(Gtk.Align.CENTER)

        self.append(self.provider_sidebar)
        self.append(self.content_stack)

        self.content_stack.connect("notify::visible-child", self._on_stack_page_changed)

        # Cache para las UI de cada proveedor para no recrearlas constantemente
        self._provider_pages_cache = {} # provider_key -> { "page": Gtk.Box, "model_list": Gtk.ListBox }

        # Icono para la página "No Selection"
        self.no_selection_icon = None
        self.no_selection_icon_target_size = 128

        self.manager.connect('api-key-changed', self._on_external_api_key_change)

    def pick_model(self, modelid):
        """
        Selecciona el modelo dado su modelid, sin importar el proveedor.
        Cambia el sidebar al proveedor correspondiente y selecciona el modelo en la lista.
        Además, emite las señales necesarias para que el contenedor actualice el UI.
        """
        for provider_key in self.manager.models_by_provider:
            models = self.manager.get_models_for_provider(provider_key)
            for model in models:
                if getattr(model, 'model_id', None) == modelid:
                    # Normalizar None a un nombre válido para el stack
                    stack_key = provider_key if provider_key is not None else "local"
                    self.content_stack.set_visible_child_name(stack_key)
                    page_ui = self._provider_pages_cache.get(provider_key)
                    if page_ui:
                        model_list = page_ui["model_list"]
                        for row in model_list:
                            if getattr(row, 'model_id', None) == modelid:
                                GLib.idle_add(model_list.select_row, row)
                                break
                    self._update_model_info_panel(provider_key, modelid)
                    # Emitir señal para que el contenedor actualice el headerbar
                    self.emit('model-selected', modelid)
                    self._update_and_emit_api_key_status(provider_key)
                    return True
        return False

    def _add_no_selection_page(self):
        """Añade la página inicial de 'Sin Selección'."""
        page_content = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=18,
                               valign=Gtk.Align.CENTER, halign=Gtk.Align.CENTER,
                               margin_top=12, margin_bottom=12, margin_start=12, margin_end=12)
        page_content.set_vexpand(True)

        info_label = Gtk.Label(wrap=True, justify=Gtk.Justification.CENTER,
                               label=_("Please select a provider from the list on the left.\nThen, choose a model from the list that appears here."))
        info_label.set_margin_bottom(12)
        page_content.append(info_label)

        # Añadir el icono "brain-augmented-symbolic"
        self.no_selection_icon = resource_manager.create_icon_widget("brain-augmented-symbolic")
        self.no_selection_icon.set_pixel_size(self.no_selection_icon_target_size)
        page_content.append(self.no_selection_icon)


        # Warning panel about API keys
        warning_bin = Adw.Bin()
        warning_bin.add_css_class("card")
        warning_bin.set_margin_top(12)
        warning_bin.set_margin_bottom(12)
        warning_bin.set_margin_start(12)
        warning_bin.set_margin_end(12)
        
        # Content inside the bin
        warning_content = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)
        warning_content.add_css_class("warning")
        warning_content.set_margin_top(12)
        warning_content.set_margin_bottom(12)
        warning_content.set_margin_start(12)
        warning_content.set_margin_end(12)
        
        # Icon and title row
        header_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=8)
        header_box.set_halign(Gtk.Align.CENTER)
        
        warning_icon = resource_manager.create_icon_widget("dialog-information")
        warning_icon.set_pixel_size(16)
        header_box.append(warning_icon)
        
        title_label = Gtk.Label()
        title_label.set_markup(f"<b>{_('Most AI models require an API key')}</b>")
        title_label.set_halign(Gtk.Align.CENTER)
        header_box.append(title_label)
        
        warning_content.append(header_box)
        
        # Description text
        desc_label = Gtk.Label(label=_("You'll need to register with each provider to obtain these authentication tokens."))
        desc_label.set_halign(Gtk.Align.CENTER)
        desc_label.set_wrap(True)
        desc_label.set_xalign(0.0)
        warning_content.append(desc_label)
        
        warning_bin.set_child(warning_content)
        
        # Wrap warning card in a clamp to reduce its size
        warning_clamp = Adw.Clamp(maximum_size=400)
        warning_clamp.set_child(warning_bin)
        page_content.append(warning_clamp)

        self.content_stack.add_titled(page_content, NO_SELECTION_KEY, _("No Selection"))

    def load_providers_and_models(self):
        """Carga los proveedores y prepara las páginas en el stack."""
        # Limpiar completamente el stack
        while self.content_stack.get_first_child():
            child = self.content_stack.get_first_child()
            self.content_stack.remove(child)
        self._provider_pages_cache.clear()
        self._add_no_selection_page()

        if not self.manager.populate_providers_and_group_models():
            placeholder_page_content = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
            placeholder_page_content.set_valign(Gtk.Align.CENTER)
            placeholder_page_content.set_halign(Gtk.Align.CENTER)
            placeholder_page_content.append(Gtk.Label(label=_("No models or providers found.")))
            self.content_stack.add_titled(placeholder_page_content, "no_providers_page", _("Error"))
            #GLib.idle_add(lambda: self.content_stack.set_visible_child_name(NO_SELECTION_KEY))
            #self._update_and_emit_api_key_status(NO_SELECTION_KEY)
            return

        sorted_providers = sorted(
            self.manager.models_by_provider.keys(),
            key=lambda p_key: self.manager.get_provider_display_name(p_key).lower()
        )
        
        for provider_key in sorted_providers:
            # Normalizar None a un nombre válido para el stack
            stack_key = provider_key if provider_key is not None else "local"
            debug_print(f"DEBUG: Agregando proveedor al stack - provider_key: '{provider_key}', stack_key: '{stack_key}', display_name: '{self.manager.get_provider_display_name(provider_key)}'")
            page_ui = self._create_page_for_provider(provider_key)
            self.content_stack.add_titled(page_ui["page"], stack_key, self.manager.get_provider_display_name(provider_key))
            self._provider_pages_cache[provider_key] = page_ui
            self._populate_model_list_for_page(provider_key, page_ui)
            # No actualizamos la tarjeta de API aquí, se hará al cambiar de página o seleccionar modelo

        # Solo forzar "No selection" si NO hay ningún modelo disponible en ningún proveedor
        any_model_available = any(
            any(getattr(model, 'model_id', None) for model in self.manager.get_models_for_provider(provider_key))
            for provider_key in self.manager.models_by_provider
        )

        #if not any_model_available:
        #    debug_print(f"DEBUG: No hay ningún modelo disponible en ningún proveedor, forzando página 'No Selection'")
            #GLib.idle_add(lambda: self.content_stack.set_visible_child_name(NO_SELECTION_KEY))
            #self._update_and_emit_api_key_status(NO_SELECTION_KEY)
        #else:
        #    debug_print(f"DEBUG: Hay al menos un modelo disponible, no se fuerza 'No Selection'")

    # --- Métodos de creación de UI para cada proveedor ---

    def _create_page_for_provider(self, provider_key: str) -> dict:
        """Crea la estructura de la página para un proveedor (API card + lista de modelos)."""
        page_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=12)
        page_box.set_margin_top(6)
        page_box.set_margin_bottom(6)
        page_box.set_margin_start(6)
        page_box.set_margin_end(6)
        page_box.set_vexpand(True)
        page_box.set_hexpand(True)

        # --- Model List ---
        model_list_scroll = Gtk.ScrolledWindow(hscrollbar_policy=Gtk.PolicyType.NEVER,
                                               vscrollbar_policy=Gtk.PolicyType.AUTOMATIC)
        model_list_scroll.set_vexpand(True)
        model_list_scroll.set_hexpand(False)
        model_list_scroll.set_size_request(250, -1)  # Fixed width for model list
        model_list = Gtk.ListBox(selection_mode=Gtk.SelectionMode.SINGLE)
        model_list.set_valign(Gtk.Align.CENTER)
        model_list.add_css_class('navigation-sidebar') 
        model_list.connect("row-activated", self._on_model_row_activated, provider_key)
        model_list_scroll.set_child(model_list)
        page_box.append(model_list_scroll)

        # --- Info Panel for selected model ---
        info_panel = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=6)
        info_panel.set_vexpand(True)
        info_panel.set_hexpand(False)
        info_panel.set_size_request(350, -1)
        info_panel.set_valign(Gtk.Align.CENTER)
        
        # Placeholder icon 
        placeholder_icon = resource_manager.create_icon_widget("brain-symbolic")
        placeholder_icon.set_pixel_size(64)
        placeholder_icon.set_halign(Gtk.Align.CENTER)
        placeholder_icon.set_valign(Gtk.Align.CENTER)
        placeholder_icon.add_css_class("dim-label")  # Atenuated color
        placeholder_icon.set_opacity(0.5)  # Static dim opacity
        info_panel.append(placeholder_icon)

        page_box.append(info_panel)

        return {"page": page_box, "model_list": model_list, "info_panel": info_panel}

    def _populate_model_list_for_page(self, provider_key: str, page_ui: dict):
        """Puebla la lista de modelos para la UI de página dada."""
        model_list_widget = page_ui["model_list"]
        self._clear_list_box(model_list_widget)

        models = self.manager.get_models_for_provider(provider_key)
        # Siempre deselecciona todo antes de poblar
        GLib.idle_add(model_list_widget.unselect_all)

        if not models:
            row = Adw.ActionRow(title=_("No models found for this provider"))
            row.set_selectable(False)
            model_list_widget.append(row)
            return

        current_model_id = self.manager.config.get('model')
        active_row = None
        for model_obj in models:
            model_id = getattr(model_obj, 'model_id', None)
            model_name = getattr(model_obj, 'name', None) or model_id
            if model_id:
                row = Adw.ActionRow(title=model_name)
                row.set_activatable(True)
                row.model_id = model_id 
                model_list_widget.append(row)
                if model_id == current_model_id:
                    active_row = row

        # Solo selecciona si hay un modelo válido
        if active_row:
            GLib.idle_add(model_list_widget.select_row, active_row)

    # --- Callbacks y manejo de estado ---

    def _on_model_row_activated(self, list_box, row, provider_key: str):
        model_id = getattr(row, 'model_id', None)
        if model_id:
            self.manager.config['model'] = model_id
            self.manager.emit('model-selected', model_id) 
            self.emit('model-selected', model_id)
            debug_print(f"WideModelSelector: Model '{model_id}' for provider '{provider_key}' selected.")
            self._update_and_emit_api_key_status(provider_key)
            self._update_model_info_panel(provider_key, model_id)

    def _on_stack_page_changed(self, stack, param):
        current_provider_key = stack.get_visible_child_name()
        debug_print(f"DEBUG: _on_stack_page_changed called - stack page changed to: '{current_provider_key}'")
        
        # Log the mapping from stack key to actual provider key
        if current_provider_key == "local":
            actual_provider = None
            debug_print(f"DEBUG: Stack key 'local' maps to provider key: None")
        elif current_provider_key == NO_SELECTION_KEY:
            actual_provider = NO_SELECTION_KEY
            debug_print(f"DEBUG: Stack key '{NO_SELECTION_KEY}' maps to provider key: {NO_SELECTION_KEY}")
        else:
            actual_provider = current_provider_key
            debug_print(f"DEBUG: Stack key '{current_provider_key}' maps to provider key: {actual_provider}")
        
        debug_print(f"DEBUG: Proveedor cambiado a: {actual_provider}")
        self._update_and_emit_api_key_status(actual_provider)

    def _show_api_key_dialog(self, provider_key: str):
        dialog = Adw.MessageDialog(
            transient_for=self.get_root(),
            modal=True,
            heading=_("Enter API Key"),
            body=f"{_('Enter the API key for')} {self.manager.get_provider_display_name(provider_key)}:",
        )
        dialog.add_response("cancel", _("Cancel"))
        dialog.add_response("set", _("Set Key"))
        dialog.set_response_appearance("set", Adw.ResponseAppearance.SUGGESTED)
        dialog.set_default_response("set")

        key_entry = Gtk.Entry(
            hexpand=True,
            placeholder_text=_("Paste your API key here")
        )
        key_entry.connect("activate", lambda entry: dialog.response("set"))
        
        content_clamp = Adw.Clamp(maximum_size=400)
        content_clamp.set_child(key_entry)
        dialog.set_extra_child(content_clamp)

        dialog.connect("response", self._on_api_key_dialog_response, key_entry)
        dialog.present()

    def _on_api_key_dialog_response(self, dialog, response_id, key_entry):
        provider_key = self._current_provider_key_for_api_dialog
        if not provider_key:
            dialog.destroy()
            return

        if response_id == "set":
            api_key = key_entry.get_text()
            if api_key: 
                self.manager.set_api_key(provider_key, api_key)
                # El _on_external_api_key_change se encargará de emitir api-key-status-changed
        dialog.destroy()
        self._current_provider_key_for_api_dialog = None

    def _on_external_api_key_change(self, manager_instance, provider_key_changed: str):
        debug_print(f"WideModelSelector: API key changed externally for provider '{provider_key_changed}', reloading models and updating UI.")
        
        # Save current selection to restore after reload
        current_provider = self.get_current_provider_key()
        current_stack_key = self.content_stack.get_visible_child_name()
        debug_print(f"WideModelSelector: Saving current selection - provider: '{current_provider}', stack_key: '{current_stack_key}'")
        
        # First reload dynamic models to get newly available models with the API key
        try:
            self.manager.reload_dynamic_models_only()
            debug_print(f"WideModelSelector: Dynamic models reloaded after API key change for '{provider_key_changed}'")
        except Exception as e:
            debug_print(f"WideModelSelector: Error reloading dynamic models: {e}")
        
        # Then update the UI
        if provider_key_changed in self._provider_pages_cache:
            page_ui = self._provider_pages_cache[provider_key_changed]
            self._populate_model_list_for_page(provider_key_changed, page_ui)
            
            # Update info panel if this provider is currently selected and a model is selected
            selected_model = self.get_selected_model_id()
            if current_provider == provider_key_changed and selected_model:
                self._update_model_info_panel(provider_key_changed, selected_model)
        
        # Restore the selection if it was the provider that changed
        if current_provider == provider_key_changed and current_stack_key != NO_SELECTION_KEY:
            debug_print(f"WideModelSelector: Restoring selection to stack_key: '{current_stack_key}'")
            GLib.idle_add(lambda: self.content_stack.set_visible_child_name(current_stack_key))
                
        self._update_and_emit_api_key_status(provider_key_changed)

    def _update_and_emit_api_key_status(self, provider_key: str):
        debug_print(f"DEBUG: _update_and_emit_api_key_status called with provider_key: '{provider_key}'")
        
        #if provider_key == NO_SELECTION_KEY or not provider_key:
        #    debug_print(f"DEBUG: Ocultando botón API key para {provider_key}")
        #    self.emit('api-key-status-changed', NO_SELECTION_KEY, False, False)
        #    return

        debug_print(f"DEBUG: Verificando status API key para proveedor: {provider_key}")
        key_status = self.manager.check_api_key_status(provider_key)
        needs_key = key_status.get('needs_key', False)
        has_key = key_status.get('has_key', False)
        debug_print(f"DEBUG: Status API key - needs_key: {needs_key}, has_key: {has_key}")
        
        if needs_key:
            debug_print(f"DEBUG: Mostrando botón API key para {provider_key}")
        else:
            debug_print(f"DEBUG: No se necesita API key para {provider_key}")
            
        self.emit('api-key-status-changed', provider_key, needs_key, has_key)

    def _clear_list_box(self, list_box: Gtk.ListBox):
        child = list_box.get_first_child()
        while child:
            next_child = child.get_next_sibling()
            list_box.remove(child)
            child = next_child

    def _update_model_info_panel(self, provider_key: str, model_id: str):
        """Actualiza el panel de información con los detalles del modelo seleccionado."""
        if provider_key not in self._provider_pages_cache:
            return
        
        page_ui = self._provider_pages_cache[provider_key]
        info_panel = page_ui["info_panel"]
        
        # Limpiar contenido actual
        child = info_panel.get_first_child()
        while child:
            next_child = child.get_next_sibling()
            info_panel.remove(child)
            child = next_child
        
        # Contenedor principal con spacing
        main_vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=18)
        main_vbox.set_valign(Gtk.Align.START)
        main_vbox.set_halign(Gtk.Align.FILL)
        main_vbox.set_margin_top(24)
        main_vbox.set_margin_start(12)
        main_vbox.set_margin_end(12)
        
        # Avatar con icono brain-symbolic
        avatar = Adw.Avatar(size=64)
        resource_manager.set_widget_icon_name(avatar, "brain-symbolic")
        avatar.set_halign(Gtk.Align.CENTER)
        avatar.set_margin_bottom(12)
        main_vbox.append(avatar)
        
        # Obtener información del modelo
        models = self.manager.get_models_for_provider(provider_key)
        model_obj = None
        for model in models:
            if getattr(model, 'model_id', None) == model_id:
                model_obj = model
                break
        
        if not model_obj:
            error_row = Adw.ActionRow()
            error_row.set_title(_("Model information not available"))
            error_row.set_subtitle(_("Unable to retrieve model details"))
            main_vbox.append(error_row)
        else:
            # ListBox para las ActionRows
            details_listbox = Gtk.ListBox()
            details_listbox.add_css_class("boxed-list")
            details_listbox.set_selection_mode(Gtk.SelectionMode.NONE)
            
            # Model ID/Name Row
            model_name = getattr(model_obj, 'name', model_id) or model_id
            model_row = Adw.ActionRow()
            model_row.set_title(_("Model"))
            if model_name != model_id:
                model_row.set_subtitle(f"{model_name} ({model_id})")
            else:
                model_row.set_subtitle(model_id)
            resource_manager.set_widget_icon_name(model_row, "brain-symbolic")
            details_listbox.append(model_row)
            
            # Aliases Row (solo si existen)
            aliases = getattr(model_obj, 'aliases', None)
            if aliases and len(aliases) > 0:
                aliases_text = ", ".join(aliases)
                aliases_row = Adw.ActionRow()
                aliases_row.set_title(_("Aliases"))
                aliases_row.set_subtitle(aliases_text)
                resource_manager.set_widget_icon_name(aliases_row, "tag-symbolic")
                details_listbox.append(aliases_row)
            
            # API Key Row
            key_status = self.manager.check_api_key_status(provider_key)
            needs_key = key_status.get('needs_key', False)
            has_key = key_status.get('has_key', False)
            
            api_key_row = Adw.ActionRow()
            api_key_row.set_title(_("API Key"))
            resource_manager.set_widget_icon_name(api_key_row, "dialog-password-symbolic")
            api_key_row.set_activatable(True)  # Make it clickable
            
            # Connect to show API key dialog when clicked
            api_key_row.connect("activated", lambda row: self._on_api_key_row_activated(provider_key))
            
            if needs_key:
                if has_key:
                    api_key_row.set_subtitle(_("Required • Set"))
                    # Icono de éxito
                    check_icon = resource_manager.create_icon_widget("padlock2-open-symbolic")
                    check_icon.add_css_class("success")
                    api_key_row.add_suffix(check_icon)
                else:
                    api_key_row.set_subtitle(_("Required • Not set"))
                    # Icono de advertencia
                    warning_icon = resource_manager.create_icon_widget("padlock2-symbolic")
                    warning_icon.add_css_class("error")
                    api_key_row.add_suffix(warning_icon)
            else:
                api_key_row.set_subtitle(_("Not required"))
                api_key_row.set_activatable(False)  # Not clickable if not needed
            
            details_listbox.append(api_key_row)
            
            # Plugin/Provider Row
            plugin_info = getattr(model_obj, 'plugin_info', None)
            provider_row = Adw.ActionRow()
            resource_manager.set_widget_icon_name(provider_row, "application-x-addon-symbolic")
            
            if plugin_info:
                plugin_name = plugin_info.get('name', _('Unknown'))
                plugin_version = plugin_info.get('version', _('Unknown'))
                provider_row.set_title(_("Plugin"))
                provider_row.set_subtitle(f"{plugin_name} v{plugin_version}")
            else:
                provider_display_name = self.manager.get_provider_display_name(provider_key)
                provider_row.set_title(_("Provider"))
                provider_row.set_subtitle(provider_display_name)
            
            details_listbox.append(provider_row)
            main_vbox.append(details_listbox)
        
        info_panel.append(main_vbox)

    def _on_api_key_row_activated(self, provider_key: str):
        """Maneja el clic en la ActionRow de API Key para mostrar el diálogo."""
        self._current_provider_key_for_api_dialog = provider_key
        self._show_api_key_dialog(provider_key)

    # --- Métodos públicos para el contenedor ---

    def get_selected_model_id(self) -> str | None:
        return self.manager.config.get('model')

    def get_current_provider_key(self) -> str | None:
        stack_key = self.content_stack.get_visible_child_name()
        # Mapear "local" de vuelta a None para compatibilidad
        return None if stack_key == "local" else stack_key

    def trigger_api_key_dialog_for_current_provider(self):
        current_provider = self.get_current_provider_key()
        if (current_provider and current_provider != NO_SELECTION_KEY):
            self._current_provider_key_for_api_dialog = current_provider
            self._show_api_key_dialog(current_provider)
        else:
            debug_print("WideModelSelector: Cannot trigger API key dialog, no valid provider selected.")

    def get_current_model_selection_status(self) -> dict:
        """
        Retorna el estado actual de la selección para validación.
        {"model_selected": bool, "needs_api_key": bool, "api_key_set": bool, "is_valid_for_next_step": bool}
        """
        provider_key = self.get_current_provider_key()
        selected_model_id = self.get_selected_model_id()

        if provider_key == NO_SELECTION_KEY or not selected_model_id:
            return {"model_selected": False, "needs_api_key": False, "api_key_set": False, "is_valid_for_next_step": False}

        key_status = self.manager.check_api_key_status(provider_key)
        needs_key = key_status.get('needs_key', False)
        has_key = key_status.get('has_key', False)

        is_valid = True if not needs_key else has_key
        return {"model_selected": True, "needs_api_key": needs_key, "api_key_set": has_key, "is_valid_for_next_step": is_valid}



"""
python313_compatibility.py - Parches de compatibilidad para Python 3.13.3
Soluciona problemas de compatibilidad con plugins LLM que usan NumPy/SciPy

Este módulo implementa parches específicos para resolver el error:
"argument docstring of add_docstring should be a str"

El problema surge porque Python 3.13.3 cambió el comportamiento de add_docstring,
pero las extensiones C de NumPy/SciPy en los plugins LLM no fueron actualizadas.
"""

import sys
import os
from functools import wraps
import warnings

DEBUG = os.environ.get('DEBUG') or False

def debug_print(*args, **kwargs):
    if DEBUG:
        print("[Python313Compat]", *args, **kwargs)

def patch_add_docstring():
    """
    Parcha la función add_docstring para ser compatible con Python 3.13.3.
    
    El problema: En Python 3.13.3, add_docstring requiere que el parámetro
    docstring sea estrictamente un str, pero algunas extensiones C pasan
    otros tipos (como bytes o None).
    
    La solución: Interceptamos llamadas a add_docstring y convertimos
    automáticamente el parámetro docstring a string si es necesario.
    """
    try:
        # Intentar parchear en numpy si está disponible
        import numpy as np
        
        # Guardar la función original
        if hasattr(np, 'add_docstring') and not hasattr(np.add_docstring, '_python313_patched'):
            original_add_docstring = np.add_docstring
            
            @wraps(original_add_docstring)
            def patched_add_docstring(obj, docstring, warn_on_python=True):
                """
                Versión parchada de numpy.add_docstring compatible con Python 3.13.3
                """
                # Convertir docstring a str si no lo es
                if docstring is not None and not isinstance(docstring, str):
                    if isinstance(docstring, bytes):
                        docstring = docstring.decode('utf-8', errors='replace')
                    else:
                        docstring = str(docstring)
                
                # Asegurar que docstring sea una cadena válida
                if docstring is None:
                    docstring = ""
                
                try:
                    return original_add_docstring(obj, docstring, warn_on_python)
                except TypeError as e:
                    if "should be a str" in str(e):
                        debug_print(f"Interceptado error add_docstring, forzando conversión: {e}")
                        # Si aún falla, usar cadena vacía
                        return original_add_docstring(obj, "", warn_on_python)
                    raise
            
            # Marcar como parchado
            patched_add_docstring._python313_patched = True
            np.add_docstring = patched_add_docstring
            debug_print("[OK] Parche numpy.add_docstring aplicado exitosamente")
            return True
            
    except ImportError:
        debug_print("NumPy no está disponible, omitiendo parche add_docstring")
        return False
    except Exception as e:
        debug_print(f"Error aplicando parche add_docstring: {e}")
        return False
    
    return False

def patch_scipy_extensions():
    """
    Parcha extensiones específicas de SciPy que pueden causar problemas
    """
    try:
        # Intentar parchear problemas conocidos de SciPy
        import scipy
        debug_print("[OK] SciPy detectado, aplicando parches preventivos")
        
        # Parche para scipy.special si está disponible
        try:
            import scipy.special
            debug_print("[OK] scipy.special importado correctamente")
        except Exception as e:
            if "add_docstring" in str(e):
                debug_print(f"Problema con scipy.special detectado: {e}")
                # Crear un módulo stub básico si es necesario
                pass
        
        return True
        
    except ImportError:
        debug_print("SciPy no está disponible, omitiendo parches")
        return False
    except Exception as e:
        debug_print(f"Error aplicando parches de SciPy: {e}")
        return False

def patch_llm_plugin_imports():
    """
    Parcha la importación de plugins LLM específicos que fallan con Python 3.13.3
    """
    known_problematic_plugins = [
        'llm_groq', 'llm_gemini', 'llm_openrouter', 
        'llm_perplexity', 'llm_anthropic', 'llm_deepseek', 'llm_grok'
    ]
    
    patched_count = 0
    
    for plugin_name in known_problematic_plugins:
        try:
            # Intentar importar el plugin
            plugin = __import__(plugin_name)
            debug_print(f"[OK] Plugin {plugin_name} importado correctamente")
            patched_count += 1
            
        except Exception as e:
            if "add_docstring" in str(e):
                debug_print(f"Plugin {plugin_name} falló con error add_docstring: {e}")
                
                # Estrategia de recuperación: intentar reimportar después de parches
                try:
                    # Aplicar parches adicionales si es necesario
                    patch_add_docstring()
                    
                    # Intentar importar nuevamente
                    if plugin_name in sys.modules:
                        del sys.modules[plugin_name]
                    
                    plugin = __import__(plugin_name)
                    debug_print(f"OK Plugin {plugin_name} recuperado exitosamente")
                    patched_count += 1
                    
                except Exception as retry_e:
                    debug_print(f"Fail: Plugin {plugin_name} no pudo ser recuperado: {retry_e}")
            else:
                debug_print(f"Fail: Plugin {plugin_name} falló por otra razón: {e}")
    
    debug_print(f"Plugins LLM procesados exitosamente: {patched_count}/{len(known_problematic_plugins)}")
    return patched_count > 0

def create_safe_llm_wrapper():
    """
    Crea un wrapper seguro para el módulo LLM que maneja errores de plugins
    """
    try:
        import llm
        
        # Verificar si necesitamos crear wrapper
        original_get_models = llm.get_models
        
        def safe_get_models():
            """Versión segura de get_models que maneja errores de plugins"""
            try:
                return list(original_get_models())
            except Exception as e:
                if "add_docstring" in str(e):
                    debug_print(f"Error en get_models interceptado: {e}")
                    debug_print("Intentando solución alternativa...")
                    
                    # Aplicar parches y reintentar
                    patch_add_docstring()
                    patch_scipy_extensions()
                    
                    try:
                        return list(original_get_models())
                    except:
                        debug_print("Devolviendo lista vacía como fallback")
                        return []
                raise
        
        # Solo reemplazar si no está ya parchado
        if not hasattr(llm.get_models, '_python313_patched'):
            llm.get_models = safe_get_models
            llm.get_models._python313_patched = True
            debug_print("OK Wrapper seguro LLM.get_models aplicado")
        
        return True
        
    except ImportError:
        debug_print("LLM no está disponible para wrapper")
        return False
    except Exception as e:
        debug_print(f"Error creando wrapper LLM: {e}")
        return False

def monkey_patch_c_extensions():
    """
    Aplica monkey patches a nivel bajo para extensiones C problemáticas
    """
    try:
        # Parche a nivel de importación para interceptar errores
        original_import = __builtins__.__import__
        
        def safe_import(name, globals=None, locals=None, fromlist=(), level=0):
            """Importación segura que intercepta errores de add_docstring"""
            try:
                return original_import(name, globals, locals, fromlist, level)
            except Exception as e:
                if "add_docstring" in str(e) and any(pkg in name for pkg in ['numpy', 'scipy', 'llm']):
                    debug_print(f"Error de importación interceptado para {name}: {e}")
                    
                    # Aplicar parches antes de reintentar
                    patch_add_docstring()
                    
                    try:
                        return original_import(name, globals, locals, fromlist, level)
                    except:
                        debug_print(f"Importación de {name} falló completamente")
                        raise
                raise
        
        # Solo aplicar el parche si Python 3.13+ y no está ya aplicado
        if sys.version_info >= (3, 13) and not hasattr(__builtins__.__import__, '_python313_patched'):
            __builtins__.__import__ = safe_import
            __builtins__.__import__._python313_patched = True
            debug_print("OK Monkey patch de importación aplicado")
            return True
        
    except Exception as e:
        debug_print(f"Error aplicando monkey patch: {e}")
        return False
    
    return False

def apply_all_patches():
    """
    Aplica todos los parches de compatibilidad para Python 3.13.3
    
    Returns:
        dict: Resultados de cada parche aplicado
    """
    if sys.version_info < (3, 13):
        debug_print("Python < 3.13 detectado, omitiendo parches de compatibilidad")
        return {"skipped": True, "reason": "python_version"}
    
    debug_print(f"Python {sys.version} detectado, aplicando parches de compatibilidad...")
    
    results = {}
    
    # Aplicar parches en orden de importancia
    try:
        results['monkey_patch'] = monkey_patch_c_extensions()
        results['add_docstring'] = patch_add_docstring()
        results['scipy'] = patch_scipy_extensions() 
        results['llm_wrapper'] = create_safe_llm_wrapper()
        results['llm_plugins'] = patch_llm_plugin_imports()
        
        success_count = sum(1 for v in results.values() if v)
        total_count = len(results)
        
        debug_print(f"Parches aplicados: {success_count}/{total_count}")
        
        if success_count > 0:
            debug_print("OK Sistema de compatibilidad Python 3.13.3 activado")
        else:
            debug_print("⚠ Ningún parche fue aplicado exitosamente")
            
    except Exception as e:
        debug_print(f"Error aplicando parches: {e}")
        results['error'] = str(e)
    
    return results

def is_python313_compatible():
    """
    Verifica si el sistema actual es compatible con los parches
    
    Returns:
        bool: True si es compatible y los parches pueden aplicarse
    """
    return (
        sys.version_info >= (3, 13) and 
        hasattr(sys, 'frozen') and 
        getattr(sys, 'frozen', False)
    )

# Auto-aplicar parches si estamos en un entorno congelado con Python 3.13+
if is_python313_compatible() and DEBUG:
    debug_print("Aplicando parches de compatibilidad automáticamente...")
    apply_all_patches()



"""
tray_applet.py - Applet de bandeja multiplataforma usando pystray y Gio para D-Bus y monitorización
"""
import sys
import os
import signal
import locale
import gettext
import threading

from .platform_utils import send_ipc_open_conversation, is_linux, is_mac
from .debug_utils import debug_print
from .db_operations import ChatHistory

try:
    import pystray
    from PIL import Image
except ImportError:
    debug_print("pystray y pillow son requeridos para el applet de bandeja.")
    sys.exit(1)

if is_linux():
    import gi
    gi.require_version('Gio', '2.0')
    from gi.repository import Gio
else:
    try:
        from watchdog.observers import Observer
        from watchdog.events import FileSystemEventHandler
    except ImportError:
        debug_print("Watchdog is required for tray applet.")
        sys.exit(1)


# --- i18n ---
APP_NAME = "gtk-llm-chat"
LOCALE_DIR = os.path.abspath(os.path.join(os.path.dirname(__file__), '..', 'po'))
lang = locale.getdefaultlocale()[0]
if lang:
    gettext.bindtextdomain(APP_NAME, LOCALE_DIR)
    gettext.textdomain(APP_NAME)
    lang_trans = gettext.translation(APP_NAME, LOCALE_DIR, languages=[lang], fallback=True)
    lang_trans.install()
    _ = lang_trans.gettext
else:
    _ = lambda s: s

# --- Icono ---
def load_icon():
    """Carga el icono para el tray. Prioriza SVG simbólico para tray, PNG para aplicación."""
    if getattr(sys, 'frozen', False):
        base_path = os.path.join(sys._MEIPASS)
    else:
        base_path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
    
    # Lista de posibles ubicaciones de iconos (priorizar SVG simbólico para tray)
    icon_paths = [
        # SVG simbólico (preferido para tray)
        os.path.join('/app', 'share', 'icons', 'hicolor', 'symbolic', 'apps', 'org.fuentelibre.gtk_llm_Chat-symbolic.svg'),
        os.path.join(base_path, 'gtk_llm_chat', 'hicolor', 'symbolic', 'apps', 'org.fuentelibre.gtk_llm_Chat-symbolic.svg'),
        # PNG 48x48 específico para el tray
        os.path.join('/app', 'share', 'icons', 'hicolor', '48x48', 'apps', 'org.fuentelibre.gtk_llm_Chat-symbolic.png'),
        os.path.join(base_path, 'gtk_llm_chat', 'hicolor', '48x48', 'apps', 'org.fuentelibre.gtk_llm_Chat-symbolic.png'),
        # PNG normal
        os.path.join('/app', 'share', 'icons', 'hicolor', '48x48', 'apps', 'org.fuentelibre.gtk_llm_Chat.png'),
        os.path.join(base_path, 'gtk_llm_chat', 'hicolor', '48x48', 'apps', 'org.fuentelibre.gtk_llm_Chat.png'),
        # Otras opciones como fallback
        os.path.join('/app', 'share', 'icons', 'hicolor', 'scalable', 'apps', 'org.fuentelibre.gtk_llm_Chat.svg'),
    ]
    
    for icon_path in icon_paths:
        debug_print(f"[TRAY ICON] Intentando cargar icono para tray: {icon_path}")
        if os.path.exists(icon_path):
            try:
                img = Image.open(icon_path)
                debug_print(f"Icono PNG cargado exitosamente: {icon_path}")
                return img
            except Exception as e:
                debug_print(f"Error cargando icono desde {icon_path}: {e}")
                continue
    
    # Si no se encuentra ningún icono, crear uno por defecto
    debug_print("No se encontró icono, creando uno por defecto")
    # Crear un icono simple de 32x32 píxeles
    try:
        img = Image.new('RGBA', (32, 32), (100, 149, 237, 255))  # Cornflower blue
        from PIL import ImageDraw
        draw = ImageDraw.Draw(img)
        draw.ellipse([8, 8, 24, 24], fill=(255, 255, 255, 255))
        return img
    except Exception as e:
        debug_print(f"Error creando icono por defecto: {e}")
        # Fallback absoluto
        return Image.new('RGBA', (32, 32), (100, 149, 237, 255))

# --- Acciones ---
def open_conversation(cid=None):
    # Asegura que el cid es string o None
    if cid is not None and not isinstance(cid, str):
        debug_print(f"[tray_applet] ADVERTENCIA: open_conversation recibió cid tipo {type(cid)}: {cid}")
        return
    send_ipc_open_conversation(cid)

def make_conv_action(cid):
    def action(icon, item):
        # Asegura que el cid es string y nunca un objeto MenuItem
        if not isinstance(cid, str):
            debug_print(f"[tray_applet] ADVERTENCIA: cid no es string, es {type(cid)}: {cid}")
            return
        open_conversation(cid)
    return action

def get_conversations_menu():
    chat_history = ChatHistory()
    items = []
    try:
        convs = chat_history.get_conversations(limit=10, offset=0)
        for conv in convs:
            label = conv['name'].strip().removeprefix("user: ")
            cid = conv['id']
            items.append(pystray.MenuItem(label, make_conv_action(cid)))
    finally:
        chat_history.close_connection()
    return items

def create_menu():
    base_items = [
        pystray.MenuItem(_("New Conversation"), lambda icon, item: open_conversation()),
        pystray.Menu.SEPARATOR,
        *get_conversations_menu(),
        pystray.Menu.SEPARATOR,
        pystray.MenuItem(_("Quit"), lambda icon, item: icon.stop())
    ]
    return pystray.Menu(*base_items)

# --- Recarga del menú usando Gio.FileMonitor ---
class DBMonitor:
    """Clase simplificada para monitorear únicamente el archivo logs.db y notificar cambios."""
    def __init__(self, db_path, on_change):
        """
        Inicializa el monitor para logs.db.
        
        Args:
            db_path: Ruta completa al archivo logs.db
            on_change: Función a llamar cuando se detectan cambios
        """
        self.db_path = os.path.abspath(db_path)
        self.db_filename = os.path.basename(db_path)
        self.on_change = on_change
        self.file_monitor = None
        self.dir_monitor = None
        self._last_mtime = 0
        self._poll_timeout_id = None
        
        debug_print(f"[tray_applet] DBMonitor: Inicializando para {self.db_path}")
        debug_print(f"[tray_applet] DBMonitor: Archivo existe: {os.path.exists(self.db_path)}")
        debug_print(f"[tray_applet] DBMonitor: Directorio padre: {os.path.dirname(self.db_path)}")
        debug_print(f"[tray_applet] DBMonitor: Directorio padre existe: {os.path.exists(os.path.dirname(self.db_path))}")
        
        # Inicializar última modificación
        if os.path.exists(self.db_path):
            self._last_mtime = os.path.getmtime(self.db_path)
            
        self._setup_monitor()
        self._start_polling_backup()
    
    def _setup_monitor(self, is_retry_after_delete=False):
        """Configura el monitor para logs.db utilizando Gio.FileMonitor."""
        # Cancelar monitores existentes
        if hasattr(self, 'file_monitor') and self.file_monitor:
            self.file_monitor.cancel()
            self.file_monitor = None
        if hasattr(self, 'dir_monitor') and self.dir_monitor:
            self.dir_monitor.cancel()
            self.dir_monitor = None

        debug_print(f"[tray_applet] DBMonitor: Configurando monitor para {self.db_path}")
        debug_print(f"[tray_applet] DBMonitor: logs.db existe: {os.path.exists(self.db_path)}")
        debug_print(f"[tray_applet] DBMonitor: is_retry_after_delete: {is_retry_after_delete}")

        # Si el archivo logs.db existe Y NO es un reintento post-delete, monitorear archivo.
        # En todos los demás casos (archivo no existe, o es reintento post-delete), monitorear directorio.
        if os.path.exists(self.db_path) and not is_retry_after_delete:
            try:
                # Verificar permisos antes de configurar el monitor
                if not os.access(self.db_path, os.R_OK):
                    debug_print(f"[tray_applet] DBMonitor: ERROR - Sin permisos de lectura en {self.db_path}")
                    self._setup_directory_monitor()
                    return
                    
                file = Gio.File.new_for_path(self.db_path)
                self.file_monitor = file.monitor_file(Gio.FileMonitorFlags.NONE, None)
                self.file_monitor.connect("changed", self._on_file_changed)
                debug_print(f"[tray_applet] DBMonitor: Monitoreando archivo logs.db directamente")
            except Exception as e:
                debug_print(f"[tray_applet] DBMonitor: Error configurando monitor de archivo: {e}")
                debug_print(f"[tray_applet] DBMonitor: Esto puede ocurrir en entornos sandbox - fallback a directorio")
                # Fallback a monitoreo de directorio
                self._setup_directory_monitor()
        else:
            self._setup_directory_monitor()
    
    def _setup_directory_monitor(self):
        """Configura monitoreo del directorio para detectar creación de logs.db."""
        dir_path = os.path.dirname(self.db_path)
        debug_print(f"[tray_applet] DBMonitor: Monitoreando directorio {dir_path} para detectar creación de logs.db")
        
        # Verificar que el directorio existe y es accesible
        if not os.path.exists(dir_path):
            debug_print(f"[tray_applet] DBMonitor: ERROR - Directorio {dir_path} no existe")
            return
            
        if not os.access(dir_path, os.R_OK):
            debug_print(f"[tray_applet] DBMonitor: ERROR - Sin permisos de lectura en {dir_path}")
            return
        
        try:
            dir_file = Gio.File.new_for_path(dir_path)
            self.dir_monitor = dir_file.monitor_directory(Gio.FileMonitorFlags.NONE, None)
            self.dir_monitor.connect("changed", self._on_dir_changed)
            self.file_monitor = None
            debug_print(f"[tray_applet] DBMonitor: Monitor de directorio configurado correctamente")
        except Exception as e:
            debug_print(f"[tray_applet] DBMonitor: Error configurando monitor de directorio: {e}")
            debug_print(f"[tray_applet] DBMonitor: Puede ser un problema de permisos o entorno sandbox")
    
    def _on_dir_changed(self, monitor, file, other_file, event_type):
        """Detecta específicamente cuando se crea el archivo logs.db."""
        filename = file.get_basename() if file else "unknown"
        
        # Log todos los eventos de directorio para diagnóstico
        event_name = "UNKNOWN"
        try:
            event_name = event_type.value_name
        except:
            event_name = str(event_type)
        
        debug_print(f"[tray_applet] DBMonitor: Evento de directorio: {filename} -> {event_name}")
        
        # Solo reaccionar si se crea logs.db (ningún otro archivo)
        if file and filename == self.db_filename and event_type == Gio.FileMonitorEvent.CREATED:
            debug_print(f"[tray_applet] DBMonitor: logs.db ha sido creado, cambiando a monitorización de archivo")
            try:
                self.dir_monitor.cancel()
                file_obj = Gio.File.new_for_path(self.db_path)
                self.file_monitor = file_obj.monitor_file(Gio.FileMonitorFlags.NONE, None)
                self.file_monitor.connect("changed", self._on_file_changed)
                debug_print(f"[tray_applet] DBMonitor: Cambio a monitor de archivo exitoso")
                self.on_change()
            except Exception as e:
                debug_print(f"[tray_applet] DBMonitor: Error cambiando a monitor de archivo: {e}")
    
    def _on_file_changed(self, monitor, file, other_file, event_type):
        """Notifica cuando se completan cambios en logs.db."""
        # Loguear todos los eventos recibidos para diagnóstico
        event_name_str = "DESCONOCIDO"
        try:
            # Intentar obtener el nombre legible del evento
            event_name_str = Gio.FileMonitorEvent(event_type).value_name 
        except ValueError:
            # Si el valor numérico no corresponde a un miembro conocido de la enumeración
            event_name_str = f"DESCONOCIDO_EVENTO_VALOR_{event_type}"

        file_path_str = file.get_path() if file else 'None'
        other_file_path_str = other_file.get_path() if other_file else 'None'
        
        debug_print(f"[tray_applet] DBMonitor: Evento {event_type} ({event_name_str}) en '{file_path_str}', otro: '{other_file_path_str}'")

        # Solo procesar eventos relevantes para logs.db
        if file and file.get_path() == self.db_path:
            # Lista de eventos que indican cambios relevantes en la base de datos
            relevant_events = [
                Gio.FileMonitorEvent.CHANGES_DONE_HINT,
                Gio.FileMonitorEvent.CHANGED,
                Gio.FileMonitorEvent.CREATED,
                Gio.FileMonitorEvent.ATTRIBUTE_CHANGED,  # Para cambios como touch
                Gio.FileMonitorEvent.PRE_UNMOUNT,       # SQLite puede hacer esto
                Gio.FileMonitorEvent.UNMOUNTED
            ]
            
            if event_type in relevant_events:
                debug_print(f"[tray_applet] DBMonitor: Detectado evento relevante {event_name_str} en logs.db - recargando menú")
                self.on_change()
            elif event_type == Gio.FileMonitorEvent.DELETED:
                debug_print(f"[tray_applet] DBMonitor: logs.db fue eliminado - volviendo a monitorear directorio")
                # Reconfigurar monitor para directorio
                self._setup_monitor(is_retry_after_delete=True)
                # Recargar menú (probablemente vacío)
                self.on_change()
            elif event_type == Gio.FileMonitorEvent.MOVED:
                debug_print(f"[tray_applet] DBMonitor: Detectado MOVED en logs.db - re-evaluando monitor")
                # SQLite a veces mueve archivos temporalmente
                self._setup_monitor(is_retry_after_delete=True)
                self.on_change()
            else:
                debug_print(f"[tray_applet] DBMonitor: Evento {event_name_str} no considerado relevante")
        else:
            debug_print(f"[tray_applet] DBMonitor: Evento en archivo diferente a logs.db, ignorado")

    def _start_polling_backup(self):
        """Inicia polling backup cada 5 segundos para detectar cambios que los monitores de archivo no capturen."""
        debug_print(f"[tray_applet] DBMonitor: Iniciando polling backup cada 5 segundos")
        
        def poll_for_changes():
            try:
                if os.path.exists(self.db_path):
                    current_mtime = os.path.getmtime(self.db_path)
                    if current_mtime != self._last_mtime:
                        debug_print(f"[tray_applet] DBMonitor: Polling detectó cambio en logs.db (mtime: {self._last_mtime} -> {current_mtime})")
                        self._last_mtime = current_mtime
                        self.on_change()
                else:
                    # Si el archivo no existe, reset mtime
                    if self._last_mtime != 0:
                        debug_print(f"[tray_applet] DBMonitor: Polling detectó que logs.db fue eliminado")
                        self._last_mtime = 0
                        self.on_change()
            except Exception as e:
                debug_print(f"[tray_applet] DBMonitor: Error en polling: {e}")
            
            # Programar próximo polling
            from gi.repository import GLib
            self._poll_timeout_id = GLib.timeout_add_seconds(5, poll_for_changes)
            return False  # No repetir automáticamente, lo controlamos manualmente
        
        # Iniciar el primer polling
        from gi.repository import GLib
        self._poll_timeout_id = GLib.timeout_add_seconds(5, poll_for_changes)

    def stop_monitoring(self):
        """Detiene todos los monitores y polling."""
        if self.file_monitor:
            self.file_monitor.cancel()
            self.file_monitor = None
        if self.dir_monitor:
            self.dir_monitor.cancel()
            self.dir_monitor = None
        if self._poll_timeout_id:
            from gi.repository import GLib
            GLib.source_remove(self._poll_timeout_id)
            self._poll_timeout_id = None
        debug_print(f"[tray_applet] DBMonitor: Monitoreo detenido")


if not is_linux():
    # --- Watchdog simplificado para Windows y macOS ---
    class DBChangeHandler(FileSystemEventHandler):
        """Manejador de eventos simplificado que solo monitorea logs.db"""
        def __init__(self, db_path, on_change):
            super().__init__()
            self.db_path = os.path.abspath(db_path)
            self.db_filename = os.path.basename(db_path)
            self.on_change = on_change
            self._last_change_time = 0
            debug_print(f"[tray_applet] DBChangeHandler: Iniciando monitor para {self.db_path}")

        def _safe_on_change(self):
            """Llama on_change de forma segura con limitación de frecuencia"""
            import time
            current_time = time.time()
            # Evitar llamadas excesivamente frecuentes (debounce de 1 segundo)
            if current_time - self._last_change_time > 1.0:
                debug_print(f"[tray_applet] DBChangeHandler: Ejecutando on_change() - recargando menú")
                self._last_change_time = current_time
                try:
                    self.on_change()
                except Exception as e:
                    debug_print(f"[tray_applet] DBChangeHandler: Error en on_change(): {e}")
            else:
                debug_print(f"[tray_applet] DBChangeHandler: on_change() omitido por debounce")

        def on_modified(self, event):
            """Detecta modificaciones de logs.db"""
            if not event.is_directory and event.src_path == self.db_path:
                debug_print(f"[tray_applet] DBChangeHandler: logs.db modificado: {event.src_path}")
                self._safe_on_change()

        def on_created(self, event):
            """Detecta creación de logs.db"""
            if not event.is_directory and event.src_path == self.db_path:
                debug_print(f"[tray_applet] DBChangeHandler: logs.db creado: {event.src_path}")
                self._safe_on_change()

        def on_moved(self, event):
            """Detecta cuando se mueve logs.db (SQLite hace esto a veces)"""
            if not event.is_directory and (event.src_path == self.db_path or event.dest_path == self.db_path):
                debug_print(f"[tray_applet] DBChangeHandler: logs.db movido de {event.src_path} a {event.dest_path}")
                self._safe_on_change()

        def on_deleted(self, event):
            """Detecta eliminación de logs.db"""
            if not event.is_directory and event.src_path == self.db_path:
                debug_print(f"[tray_applet] DBChangeHandler: logs.db eliminado: {event.src_path}")
                self._safe_on_change()
            import time
            current_time = time.time()
            # Evita llamadas múltiples en menos de 1 segundo
            if current_time - self._last_change_time < 1.0:
                return
            self._last_change_time = current_time
            
            try:
                debug_print(f"[tray_applet] Detectado cambio en logs.db, actualizando menú")
                self.on_change()
            except Exception as e:
                debug_print(f"[tray_applet] Error al actualizar menú: {e}")

        def on_modified(self, event):
            # Solo reaccionar si es logs.db
            if (not event.is_directory and 
                os.path.basename(event.src_path) == self.db_filename):
                self._safe_on_change()

        def on_created(self, event):
            # Solo reaccionar si es logs.db
            if (not event.is_directory and 
                os.path.basename(event.src_path) == self.db_filename):
                debug_print(f"[tray_applet] logs.db ha sido creado")
                self._safe_on_change()

        def on_moved(self, event):
            # SQLite a veces usa operaciones de movimiento durante transacciones
            if (not event.is_directory and 
                hasattr(event, 'dest_path') and event.dest_path and
                os.path.basename(event.dest_path) == self.db_filename):
                self._safe_on_change()

# --- Señal para salir limpio ---
def on_quit_signal(sig, frame):
    debug_print(_("\nClosing application..."))
    sys.exit(0)

signal.signal(signal.SIGINT, on_quit_signal)

# --- Main ---
def main():
    """Main function for the tray applet."""
    # Import ensure_single_instance here to avoid circular dependencies if platform_utils imports tray_applet
    from .platform_utils import ensure_single_instance
    # Ensure this is the only applet instance.
    # The returned object must be kept in scope to maintain the lock.
    _applet_instance_lock = ensure_single_instance("gtk_llm_applet")

    # Obtener el path de la base de datos
    from .platform_utils import ensure_user_dir_exists, debug_database_monitoring
    from .debug_utils import DEBUG
    
    # Ejecutar diagnóstico en modo DEBUG
    if DEBUG:
        debug_database_monitoring()
    
    user_dir = ensure_user_dir_exists()
    db_path = os.path.join(user_dir, "logs.db")
    
    # Diagnóstico detallado del path de la base de datos
    debug_print(f"[tray_applet] === DIAGNÓSTICO DE MONITOREO ===")
    debug_print(f"[tray_applet] user_dir (llm.user_dir()): {user_dir}")
    debug_print(f"[tray_applet] PATH ABSOLUTO DE logs.db MONITORIZADO: {db_path}")
    debug_print(f"[tray_applet] logs.db existe: {os.path.exists(db_path)}")
    debug_print(f"[tray_applet] user_dir existe: {os.path.exists(user_dir)}")
    debug_print(f"[tray_applet] is_frozen: {getattr(sys, 'frozen', False)}")
    debug_print(f"[tray_applet] FLATPAK_ID: {os.environ.get('FLATPAK_ID', 'None')}")
    debug_print(f"[tray_applet] LLM_USER_PATH: {os.environ.get('LLM_USER_PATH', 'None')}")
    
    if os.path.exists(user_dir):
        try:
            contents = os.listdir(user_dir)
            debug_print(f"[tray_applet] Contenido de user_dir ({len(contents)} elementos):")
            for item in contents[:5]:  # Mostrar primeros 5
                debug_print(f"[tray_applet]   {item}")
            if len(contents) > 5:
                debug_print(f"[tray_applet]   ... y {len(contents) - 5} más")
        except Exception as e:
            debug_print(f"[tray_applet] Error listando user_dir: {e}")
    
    # Inicializar el icon de bandeja
    if is_linux():
        # La forma correcta y moderna: pasar el nombre.
        # El escritorio lo encontrará y le aplicará el tema correcto.
        icon_spec = "org.fuentelibre.gtk_llm_Chat-symbolic" 
    else:
        icon_spec = load_icon()

    icon = pystray.Icon(
        "org.fuentelibre.gtk_llm_Chat",
        icon_spec,
        _("LLM Conversations"))

    icon_id = "org.fuentelibre.gtk_llm_Chat" if os.path.exists('/.flatpak-info') else "LLMChatApplet"
    icon = pystray.Icon(icon_id, icon_spec, _(u"LLM Conversations"))
    
    def reload_menu():
        """Recarga el menú con las conversaciones actualizadas desde logs.db"""
        debug_print(f"[tray_applet] reload_menu() llamado - recargando conversaciones desde {db_path}")
        icon.menu = create_menu()
    
    # Verificar que logs.db existe antes de continuar
    if not os.path.exists(db_path):
        debug_print(f"[tray_applet] ADVERTENCIA: No se encontró logs.db en {db_path}")
        debug_print(f"[tray_applet] El applet solo debe iniciarse después de que el usuario haya configurado un modelo")
        
        # Mostrar un menú básico para que el usuario pueda iniciar una nueva conversación
        # (que creará logs.db) o salir
        icon.menu = pystray.Menu(
            pystray.MenuItem(_("New Conversation"), lambda icon, item: open_conversation()),
            pystray.Menu.SEPARATOR,
            pystray.MenuItem(_("Quit"), lambda icon, item: icon.stop())
        )
    else:
        # Si logs.db existe, cargar las conversaciones
        debug_print(f"[tray_applet] Encontrado logs.db, cargando conversaciones desde {db_path}")
        icon.menu = create_menu()
    
    # Configurar la monitorización de logs.db de forma simplificada
    debug_print(f"[tray_applet] Configurando monitorización de logs.db...")
    if is_linux():
        debug_print(f"[tray_applet] Iniciando monitorización Gio (Linux) para {db_path}")
        # Gio requiere loop GLib, ejecutar en un hilo aparte
        def gio_loop():
            try:
                db_monitor = DBMonitor(db_path, reload_menu)
                debug_print(f"[tray_applet] DBMonitor inicializado correctamente")
                from gi.repository import GLib
                loop = GLib.MainLoop()
                debug_print(f"[tray_applet] Iniciando GLib.MainLoop para monitorización")
                loop.run()
            except Exception as e:
                debug_print(f"[tray_applet] Error en gio_loop: {e}")
                import traceback
                traceback.print_exc()
        threading.Thread(target=gio_loop, daemon=True).start()
    else:
        platform_name = "macOS" if is_mac() else "Windows"
        debug_print(f"[tray_applet] Iniciando monitorización Watchdog ({platform_name}) para {db_path}")
        try:
            event_handler = DBChangeHandler(db_path, reload_menu)
            observer = Observer()
            # Solo monitoreamos el directorio que contiene logs.db
            db_dir = os.path.dirname(db_path)
            debug_print(f"[tray_applet] Monitoreando directorio: {db_dir}")
            observer.schedule(event_handler, db_dir, recursive=False)
            observer.daemon = True
            observer.start()
            debug_print(f"[tray_applet] Observer iniciado correctamente")
        except Exception as e:
            debug_print(f"[tray_applet] Error configurando Watchdog: {e}")
            import traceback
            traceback.print_exc()
    
    debug_print(f"[tray_applet] Iniciando icon.run() - el applet debería estar visible en la bandeja")
    # Ejecutar el icono de bandeja
    icon.run()

if __name__ == '__main__':
    # ensure_single_instance is now called within main()
    main()



"""
debug_utils.py - helpers puros sin dependencias de GTK ni gi
"""
import os

DEBUG = os.environ.get('DEBUG') or False

def debug_print(*args, **kwargs):
    if DEBUG:
        print(*args, **kwargs)



import gi
gi.require_version('Gtk', '4.0')
gi.require_version('Adw', '1')
import time
from gi.repository import Gtk, Adw, Gio, Gdk, GLib
import os
import threading
from .debug_utils import debug_print
from .resource_manager import resource_manager
from .style_manager import style_manager
from .chat_application import _


class WelcomeWindow(Adw.ApplicationWindow):
    def __init__(self, app, on_welcome_finished=None):
        super().__init__(application=app)
        self._on_welcome_finished = on_welcome_finished
        self.set_default_size(900, 700)
        self.set_size_request(700, 500)  # Tamaño mínimo razonable para evitar colapsos
        self.panel_titles = ["", _("Tray applet"), _("Default Model"), ""]
        self.config_data = {}
        self._finish_clicked = False # Bandera para controlar el flujo de cierre

        # Aplicar clase principal (configuración de recursos se hace más tarde)
        style_manager.apply_to_widget(self, "main-container")

        import sys
        self.header_bar = Adw.HeaderBar()
        self.header_bar.set_show_end_title_buttons(True)
        if sys.platform == 'darwin':
            def _apply_native_controls():
                style_manager.apply_macos_native_window_controls(self.header_bar)
                return False  # Ejecutar solo una vez
            GLib.idle_add(_apply_native_controls)

        # Conectar la señal realize para iniciar la carga de modelos automáticamente
        self.connect('realize', self._on_realize)
        # Conectar la señal show para configurar recursos de forma segura
        self.connect('show', self._on_window_show)
        
        root_vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
        root_vbox.set_hexpand(True)
        root_vbox.set_vexpand(True)
        root_vbox.set_size_request(700, 500)
        self.prev_button = Gtk.Button()
        resource_manager.set_widget_icon_name(self.prev_button, "go-previous-symbolic")
        self.prev_button.add_css_class("flat")
        self.prev_button.set_size_request(100, 40)
        self.prev_button.connect('clicked', self.on_prev_clicked)
        self.header_bar.pack_start(self.prev_button)

        self.next_button = Gtk.Button(label=_("Next"))
        self.next_button.add_css_class("suggested-action")
        self.next_button.set_size_request(100, 40)
        self.next_button.connect('clicked', self.on_next_clicked)
        self.header_bar.pack_end(self.next_button)

        self.start_chatting_button = Gtk.Button(label=_("New Conversation"))
        self.start_chatting_button.add_css_class("suggested-action")
        self.start_chatting_button.set_halign(Gtk.Align.CENTER)
        self.start_chatting_button.set_size_request(160, 40)
        self.start_chatting_button.connect('clicked', self.on_finish_clicked)

        self.api_key_button = Gtk.Button()
        self.api_key_button.set_size_request(120, 40)  # Tamaño mínimo fijo
        self.api_key_button.connect('clicked', self._on_api_key_button_clicked)
        self.api_key_button_packed = False

        self.loading_spinner = Adw.Spinner()
        self.loading_spinner.set_size_request(32, 32)  # Spinner más grande para visibilidad
        self.loading_spinner_packed = False

        root_vbox.append(self.header_bar)
        self.set_content(root_vbox)

        self.carousel = Adw.Carousel()
        self.carousel.set_vexpand(True)
        self.carousel.set_hexpand(True)
        self.carousel.set_halign(Gtk.Align.FILL)
        self.carousel.set_orientation(Gtk.Orientation.HORIZONTAL)
        self.carousel.set_interactive(False)
        self.carousel.set_size_request(700, 450)

        def make_clamped(child, fill_vertical=False, halign=Gtk.Align.CENTER):
            clamp = Adw.Clamp()
            clamp.set_maximum_size(800)
            clamp.set_tightening_threshold(400)
            clamp.set_hexpand(True)
            clamp.set_halign(halign)
            clamp.set_vexpand(True)
            clamp.set_valign(Gtk.Align.FILL if fill_vertical else Gtk.Align.CENTER)
            clamp.set_child(child)
            return clamp

        # Panel 1: Bienvenida
        page1 = Adw.StatusPage()
        page1.set_hexpand(True)
        page1.set_halign(Gtk.Align.FILL)
        page1.set_size_request(700, 450)
        
        # Usar resource_manager para cargar la imagen
        vbox1 = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=18)
        vbox1.set_valign(Gtk.Align.CENTER)
        vbox1.set_halign(Gtk.Align.CENTER)
        vbox1.set_hexpand(False)
        vbox1.set_vexpand(False)
        # No forzar size_request aquí, dejar que el clamp lo limite
        
        # Usar resource_manager para cargar el icono de la aplicación con tamaño grande
        app_image = resource_manager.create_icon_widget("org.fuentelibre.gtk_llm_Chat", 256)
        app_image.set_valign(Gtk.Align.CENTER)
        app_image.set_halign(Gtk.Align.CENTER)
        app_image.set_size_request(256, 256)
        vbox1.append(app_image)
        
        page1.set_title(_("Own the conversation."))
        page1.set_description(_("Use any model you want. Your conversations are stored locally."))
        panel1_desc_label2 = Gtk.Label(label=_("This wizard will guide you through the initial setup"))
        panel1_desc_label2.set_wrap(True)
        panel1_desc_label2.set_justify(Gtk.Justification.CENTER)
        panel1_desc_label2.set_halign(Gtk.Align.CENTER)
        panel1_desc_label2.set_max_width_chars(50)
        vbox1.append(panel1_desc_label2)
        self.start_button = Gtk.Button(label=_("Start"))
        self.start_button.add_css_class("suggested-action")
        self.start_button.set_halign(Gtk.Align.CENTER)
        self.start_button.set_valign(Gtk.Align.CENTER)
        self.start_button.set_hexpand(True)
        self.start_button.connect('clicked', self.on_start_clicked)
        vbox1.append(self.start_button)
        # Usar un Gtk.Box vertical expandible para centrar vbox1 verticalmente
        centering_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        centering_box.set_hexpand(True)
        centering_box.set_vexpand(True)
        centering_box.append(Gtk.Box(vexpand=True))  # Espaciador arriba
        centering_box.append(vbox1)
        centering_box.append(Gtk.Box(vexpand=True))  # Espaciador abajo
        page1.set_child(make_clamped(centering_box))

        # Panel 2: Applet de bandeja
        page2_vbox_content = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)
        page2_vbox_content.set_valign(Gtk.Align.CENTER)
        page2_vbox_content.set_halign(Gtk.Align.CENTER)
        page2_vbox_content.set_hexpand(False)
        page2_vbox_content.set_vexpand(False)
        # No forzar size_request aquí, dejar que el clamp lo limite
        
        self.panel2_app_icon_target_size = 64
        # Usar resource_manager para crear el icono
        self.panel2_app_icon = resource_manager.create_icon_widget("org.fuentelibre.gtk_llm_Chat-symbolic", 1)
        self.panel2_app_icon.set_opacity(0.0)
        self.panel2_app_icon.set_size_request(1, 1)
        self.panel2_app_icon.set_halign(Gtk.Align.CENTER)
        self.panel2_app_icon.set_margin_bottom(12)
        page2_vbox_content.append(self.panel2_app_icon)
        panel2_desc_label = Gtk.Label(label=_("Access conversations from the convenience of your system tray"))
        panel2_desc_label.set_wrap(True)
        panel2_desc_label.set_justify(Gtk.Justification.CENTER)
        panel2_desc_label.set_halign(Gtk.Align.CENTER)
        panel2_desc_label.set_max_width_chars(50)
        page2_vbox_content.append(panel2_desc_label)
        panel2_desc_label2 = Gtk.Label(label=_("Would you like to start the applet with your session?"))
        panel2_desc_label2.set_wrap(True)
        panel2_desc_label2.set_justify(Gtk.Justification.CENTER)
        panel2_desc_label2.set_halign(Gtk.Align.CENTER)
        panel2_desc_label2.set_max_width_chars(50)
        page2_vbox_content.append(panel2_desc_label2)
        self.tray_group = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=8)
        self.tray_group.set_hexpand(True)
        self.tray_group.set_halign(Gtk.Align.CENTER)
        self.tray_group.set_margin_top(12)
        self.tray_radio1 = Gtk.CheckButton(label=_("Yes, with my session"))
        self.tray_radio2 = Gtk.CheckButton(label=_("No, only when I start the app"))
        self.tray_radio1.set_group(self.tray_radio2)
        self._initialize_tray_options()
        self.tray_radio1.connect('toggled', self._on_tray_option_changed)
        self.tray_radio2.connect('toggled', self._on_tray_option_changed)
        self.tray_group.append(self.tray_radio1)
        self.tray_group.append(self.tray_radio2)
        page2_vbox_content.append(self.tray_group)
        panel2_app_animation_target = Adw.CallbackAnimationTarget.new(self._animate_panel2_app_icon_callback)
        self.panel2_app_animation = Adw.TimedAnimation.new(self.panel2_app_icon, 0.0, 1.0, 700, panel2_app_animation_target)
        self.panel2_app_animation.set_easing(Adw.Easing.EASE_OUT_EXPO)
        self.panel2_app_animation_played = False

        # Panel 3: Selección de modelo y API key
        page3_container = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        page3_container.set_hexpand(True)
        page3_container.set_halign(Gtk.Align.FILL)
        page3_container.set_vexpand(True)
        page3_container.set_valign(Gtk.Align.FILL)
        page3_container.set_size_request(700, 450)
        panel3_inner_vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)
        panel3_inner_vbox.set_valign(Gtk.Align.FILL)
        panel3_inner_vbox.set_halign(Gtk.Align.FILL)
        panel3_inner_vbox.set_hexpand(True)
        panel3_inner_vbox.set_vexpand(True)
        self.model_manager = None
        self.model_selector = None
        self._models_loaded = False
        self._model_selector_created = False
        self.app = app
        self.panel3_placeholder = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        self.panel3_placeholder.set_vexpand(True)
        self.panel3_placeholder.set_hexpand(True)
        self.panel3_placeholder.set_valign(Gtk.Align.CENTER)
        self.panel3_placeholder.set_halign(Gtk.Align.CENTER)
        placeholder_label = Gtk.Label(label=_("Loading model selection..."))
        placeholder_label.add_css_class("dim-label")
        self.panel3_placeholder.append(placeholder_label)
        panel3_inner_vbox.append(self.panel3_placeholder)
        page3_container.append(make_clamped(panel3_inner_vbox, fill_vertical=True, halign=Gtk.Align.FILL))

        # Panel 4: Listo para comenzar
        page4 = Adw.StatusPage()
        page4.set_hexpand(True)
        page4.set_halign(Gtk.Align.FILL)
        page4.set_title(_("Ready to start!"))
        page4.set_size_request(700, 450)
        page4_vbox_content = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)
        page4_vbox_content.set_valign(Gtk.Align.CENTER)
        page4_vbox_content.set_halign(Gtk.Align.CENTER)
        page4_vbox_content.set_hexpand(True)
        checkmark_icon = resource_manager.create_icon_widget("checkmark-symbolic")
        checkmark_icon.set_pixel_size(128)
        checkmark_icon.set_halign(Gtk.Align.CENTER)
        checkmark_icon.set_margin_bottom(24)
        checkmark_icon.add_css_class("success")
        page4_vbox_content.append(checkmark_icon)
        page4.set_child(make_clamped(page4_vbox_content))

        # Centrado vertical para el panel 2
        centering_box2 = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
        centering_box2.set_hexpand(True)
        centering_box2.set_vexpand(True)
        centering_box2.append(Gtk.Box(vexpand=True))  # Espaciador arriba
        centering_box2.append(page2_vbox_content)
        centering_box2.append(Gtk.Box(vexpand=True))  # Espaciador abajo

        self.carousel.append(page1)
        self.carousel.append(make_clamped(centering_box2, fill_vertical=False))
        self.carousel.append(make_clamped(page3_container, fill_vertical=True))
        self.carousel.append(page4)
        self.indicator_dots = Adw.CarouselIndicatorDots(carousel=self.carousel)
        self.indicator_dots.set_halign(Gtk.Align.CENTER)
        self.indicator_dots.set_valign(Gtk.Align.END)
        self.indicator_dots.set_margin_top(6)
        self.indicator_dots.set_margin_bottom(18)
        self.carousel.set_hexpand(True)
        self.carousel.set_halign(Gtk.Align.FILL)
        root_vbox.append(self.carousel)
        root_vbox.append(self.indicator_dots)
        self.carousel.connect("page-changed", self.on_page_changed)
        self.update_navigation_buttons()
        self.on_page_changed(self.carousel, self.carousel.get_position())
        key_controller = Gtk.EventControllerKey()
        key_controller.set_propagation_phase(Gtk.PropagationPhase.CAPTURE)
        key_controller.connect("key-pressed", self.on_key_pressed)
        self.add_controller(key_controller)
        
        # Conectar señal de cierre de ventana para terminar la aplicación si se cierra sin completar
        self.connect('close-request', self._on_window_close_request)

    def on_start_clicked(self, button):
        page_to_scroll_to = self.carousel.get_nth_page(1)
        if page_to_scroll_to:
            self.carousel.scroll_to(page_to_scroll_to, True)

    def on_prev_clicked(self, button):
        current_page_idx = int(round(self.carousel.get_position()))
        if current_page_idx > 0:
            prev_page_widget = self.carousel.get_nth_page(current_page_idx - 1)
            if prev_page_widget:
                self.carousel.scroll_to(prev_page_widget, True)

    def on_next_clicked(self, button):
        current_page_idx = int(round(self.carousel.get_position()))
        n_pages = self.carousel.get_n_pages()
        if current_page_idx == 2:
            if self.model_selector:
                status = self.model_selector.get_current_model_selection_status()
                if not status["is_valid_for_next_step"]:
                    debug_print("DEBUG: Cannot proceed, model/API key selection is not valid.")
                    return
            else:
                debug_print("DEBUG: Model selector not ready yet.")
                return
        if current_page_idx < n_pages - 1:
            next_page_widget = self.carousel.get_nth_page(current_page_idx + 1)
            if next_page_widget:
                self.carousel.scroll_to(next_page_widget, True)

    def on_page_changed(self, carousel, page_index):
        self.update_navigation_buttons()
        current_page_as_int = int(round(page_index))
        if 0 <= current_page_as_int < len(self.panel_titles):
            self.set_title(self.panel_titles[current_page_as_int])
        else:
            self.set_title("")
        if current_page_as_int == 1 and not self.panel2_app_animation_played:
            if hasattr(self, 'panel2_app_animation'):
                self.panel2_app_animation.play()
                self.panel2_app_animation_played = True
        if current_page_as_int == 2:
            self.prev_button.set_can_focus(False)
            if self.model_selector and hasattr(self.model_selector, 'provider_sidebar') and self.model_selector.provider_sidebar:
                def attempt_focus_and_reenable_prev_button():
                    if int(round(self.carousel.get_position())) == 2:
                        if self.model_selector.provider_sidebar.get_realized() and self.model_selector.provider_sidebar.get_mapped():
                            self.model_selector.provider_sidebar.grab_focus()
                    if self.prev_button.is_sensitive():
                        self.prev_button.set_can_focus(True)
                    return GLib.SOURCE_REMOVE
                GLib.timeout_add(100, attempt_focus_and_reenable_prev_button)
        else:
            if self.prev_button.is_sensitive():
                self.prev_button.set_can_focus(True)
            else:
                self.prev_button.set_can_focus(False)


    def on_key_pressed(self, controller, keyval, keycode, state):
        if not (keyval == Gdk.KEY_Return or keyval == Gdk.KEY_KP_Enter):
            return False
        current_page_idx = int(round(self.carousel.get_position()))
        n_pages = self.carousel.get_n_pages()
        if current_page_idx == 0:
            self.on_start_clicked(None)
            return True
        elif current_page_idx == n_pages - 1:
            self.on_finish_clicked(None)
            return True
        else:
            self.on_next_clicked(None)
            return True

    def update_navigation_buttons(self):
        idx = self.carousel.get_position()
        current_page_idx = int(round(idx))
        n = self.carousel.get_n_pages()
        self.prev_button.set_sensitive(current_page_idx > 0)

        # Página 0: Bienvenida
        if current_page_idx == 0:
            self.prev_button.set_visible(False)
            self.next_button.set_visible(False)
            self._ensure_api_key_button_removed()
            return

        # Última página: Listo para comenzar
        if current_page_idx == n - 1:
            self.prev_button.set_visible(True)
            self.next_button.set_visible(False)
            if self.start_chatting_button.get_parent() != self.header_bar:
                self.header_bar.pack_end(self.start_chatting_button)
            self.start_chatting_button.set_visible(True)
            self._ensure_api_key_button_removed()
            return

        # Otras páginas
        self.prev_button.set_visible(True)
        self.next_button.set_visible(True)
        self.start_chatting_button.set_visible(False)

        # Página de selección de modelo
        if current_page_idx == 2:
            if not self._models_loaded or not self._model_selector_created:
                if not self.loading_spinner_packed:
                    self.header_bar.pack_start(self.loading_spinner)
                    self.loading_spinner_packed = True
                self.loading_spinner.set_visible(True)
                self.next_button.set_sensitive(False)
                self._ensure_api_key_button_removed()
                return
            else:
                self._ensure_loading_spinner_removed()

            # Si hay selector de modelo, usar su status
            if self.model_selector:
                status = self.model_selector.get_current_model_selection_status()
                current_provider = self.model_selector.get_current_provider_key()
                debug_print(f"Debug update_navigation_buttons: current_provider={current_provider}")

                # Mostrar u ocultar botón de API key según status
                self._update_api_key_button(current_provider)
                # El botón Next se habilita si hay un modelo seleccionado válido
                self.next_button.set_sensitive(status["is_valid_for_next_step"])
            else:
                self.next_button.set_sensitive(False)
                self._ensure_api_key_button_removed()
        else:
            self.next_button.set_sensitive(True)
            self._ensure_api_key_button_removed()
            self._ensure_loading_spinner_removed()

    def _update_api_key_button(self, current_provider):
        """Muestra u oculta el botón de API key según el estado del proveedor actual."""
        provider_needs_key = False
        provider_has_key = False
        models_count = 0
        if current_provider and current_provider != "no_selection" and hasattr(self.model_manager, 'check_api_key_status'):
            key_status = self.model_manager.check_api_key_status(current_provider)
            provider_needs_key = key_status.get('needs_key', False)
            provider_has_key = key_status.get('has_key', False)
            if hasattr(self.model_manager, 'models_by_provider') and current_provider in self.model_manager.models_by_provider:
                models_count = len(self.model_manager.models_by_provider[current_provider])
            # Heurística extra: si está en el mapeo de needs_key pero no lo reporta, forzar needs_key
            if not provider_needs_key and hasattr(self.model_manager, '_provider_to_needs_key'):
                if current_provider in self.model_manager._provider_to_needs_key:
                    needs_key_name = self.model_manager._provider_to_needs_key[current_provider]
                    if needs_key_name and needs_key_name != "":
                        provider_needs_key = True
                        try:
                            import llm
                            key_value = llm.get_key(needs_key_name)
                            key_exists = key_value is not None and key_value.strip() != ""
                            provider_has_key = key_exists and models_count > 0
                        except Exception:
                            provider_has_key = False
        if provider_needs_key:
            debug_print(f"Debug: Mostrando botón API key para {current_provider}")
            if not self.api_key_button_packed:
                if self.api_key_button.get_parent() is not None:
                    self.api_key_button.get_parent().remove(self.api_key_button)
                self.api_key_button.set_visible(False)
                self.header_bar.pack_start(self.api_key_button)
                self.api_key_button_packed = True
            button_text = _("Set API Key") if not provider_has_key else _("Change API Key")
            self.api_key_button.set_label(button_text)
            self.api_key_button.get_style_context().remove_class("suggested-action")
            self.api_key_button.get_style_context().remove_class("destructive-action")
            if not provider_has_key:
                self.api_key_button.get_style_context().add_class("suggested-action")
            else:
                self.api_key_button.get_style_context().add_class("destructive-action")
            self.api_key_button.set_visible(True)
        else:
            debug_print(f"Debug: Ocultando botón API key para {current_provider}")
            self._ensure_api_key_button_removed()

    def _ensure_api_key_button_removed(self):
        if self.api_key_button_packed:
            try:
                if self.api_key_button.get_parent() == self.header_bar:
                    self.header_bar.remove(self.api_key_button)
                self.api_key_button_packed = False
            except Exception as e:
                debug_print(f"Error removiendo botón API key: {e}")
                self.api_key_button_packed = False
        self.api_key_button.set_visible(False)

    def _ensure_loading_spinner_removed(self):
        if self.loading_spinner_packed:
            try:
                if self.loading_spinner.get_parent() == self.header_bar:
                    self.header_bar.remove(self.loading_spinner)
                self.loading_spinner_packed = False
            except Exception as e:
                debug_print(f"Error removiendo spinner: {e}")
                self.loading_spinner_packed = False
        self.loading_spinner.set_visible(False)

    def on_finish_clicked(self, button):
        self._finish_clicked = True # Marcar que el flujo se completó
        self.close()
        if self._on_welcome_finished:
            self._on_welcome_finished(self.get_configuration())

    def _on_window_close_request(self, window):
        """Maneja el cierre de la ventana de bienvenida."""
        if not self._finish_clicked:
            debug_print("Ventana de bienvenida cerrada ANTES de completar el asistente, terminando aplicación")
            self.app.quit() # Terminar la aplicación si el flujo no se completó
        else:
            debug_print("Ventana de bienvenida cerrada DESPUÉS de completar el asistente (por self.close()). La app continúa.")
        return False # Permitir que la ventana se cierre en cualquier caso


    def _on_model_selected(self, selector, model_id):
        self.config_data['model'] = model_id
        provider_key = None
        # Intentar obtener el provider_key usando llm_client si está disponible
        llm_client = getattr(self.app, 'llm_client', None)
        if llm_client and hasattr(llm_client, 'get_provider_for_model'):
            provider_key = llm_client.get_provider_for_model(model_id)
        else:
            # Fallback: buscar en los modelos cargados
            try:
                import llm
                all_models = llm.get_models()
                for m in all_models:
                    if getattr(m, 'model_id', None) == model_id:
                        provider_key = getattr(m, 'needs_key', None)
                        break
            except Exception:
                provider_key = None
        # Consultar el estado de la API key
        needs_key = False
        has_key = False
        if provider_key is not None and self.model_manager and hasattr(self.model_manager, 'check_api_key_status'):
            key_status = self.model_manager.check_api_key_status(provider_key)
            needs_key = key_status.get('needs_key', False)
            has_key = key_status.get('has_key', False)
        # Solo setear modelo por defecto si no requiere clave o si la clave está presente
        if not needs_key or has_key:
            try:
                import llm
                llm.set_default_model(model_id)

                # Forzar la creación de logs.db para evitar ejecuciones repetidas del asistente
                try:
                    # Usamos la lógica ya implementada en ChatHistory
                    from .db_operations import ChatHistory
                    
                    # Solo necesitamos inicializar ChatHistory y llamar a _ensure_db_exists()
                    temp_history = ChatHistory()
                    temp_history._ensure_db_exists()  # Esto aplica las migraciones si el archivo no existe
                    self.app._needs_initial_setup = False
                    debug_print("Base de datos logs.db creada correctamente")
                except Exception as db_err:
                    debug_print(f"Error forzando creación de BD: {db_err}")
                
                debug_print(f"Modelo por defecto configurado: {model_id}")
            except Exception as e:
                debug_print(f"Error configurando modelo por defecto: {e}")
            
            from .platform_utils import spawn_tray_applet
            spawn_tray_applet({})
        self.update_navigation_buttons()

    def save_configuration(self):
        if self.tray_radio1.get_active():
            self.config_data['tray_startup'] = 'session'
        elif self.tray_radio2.get_active():
            self.config_data['tray_startup'] = 'application'
        else:
            self.config_data['tray_startup'] = 'never'
        debug_print(f"Configuration saved: {self.config_data}")

    def get_configuration(self):
        return self.config_data.copy()

    def _on_api_key_button_clicked(self, button):
        if self.model_selector:
            self.model_selector.trigger_api_key_dialog_for_current_provider()

    def _on_api_key_status_changed(self, selector, provider_key, needs_key, has_key):
        debug_print(f"Debug: API key status changed - provider: {provider_key}, needs: {needs_key}, has: {has_key}")

        # Si se configuró una API key (needs_key=True y has_key=True), recargar solo los modelos dinámicos
        # sin cambiar el stack para evitar bucles infinitos
        if needs_key and has_key and self.model_selector:
            debug_print(f"Debug: API key configurada para {provider_key}, recargando solo modelos dinámicos...")
            try:
                # Recargar modelos dinámicos directamente sin tocar la UI del stack
                self.model_selector.manager.reload_dynamic_models_only()
                
                # Actualizar solo la página del proveedor que cambió
                if provider_key in self.model_selector._provider_pages_cache:
                    page_ui = self.model_selector._provider_pages_cache[provider_key]
                    self.model_selector._populate_model_list_for_page(provider_key, page_ui)
                    debug_print(f"Debug: Lista de modelos actualizada para proveedor {provider_key}")
                
            except Exception as e:
                debug_print(f"Error recargando modelos dinámicos: {e}")

        # Actualizar botones en cualquier caso
        if int(round(self.carousel.get_position())) == 2:
            self.update_navigation_buttons()

    def _animate_panel2_app_icon_callback(self, value):
        self.panel2_app_icon.set_opacity(value)
        end_size = self.panel2_app_icon_target_size
        current_size = int(end_size * value)
        self.panel2_app_icon.set_size_request(current_size, current_size)
        self.panel2_app_icon.set_pixel_size(current_size)

    def start_lazy_loading(self):
        if not self._models_loaded:
            if int(round(self.carousel.get_position())) == 2:
                if not self.loading_spinner_packed:
                    self.header_bar.pack_start(self.loading_spinner)
                    self.loading_spinner_packed = True
                self.loading_spinner.set_visible(True)
            GLib.timeout_add(200, self._start_background_loading)

    def _start_background_loading(self):
        threading.Thread(
            target=self._load_models_in_thread,
            daemon=True,
            name="ModelLoader"
        ).start()
        return False

    def _load_models_in_thread(self):
        try:
            from .wide_model_selector import WideModelSelector, NO_SELECTION_KEY
            from .model_selection import ModelSelectionManager
            GLib.idle_add(self._create_model_selector_and_replace_placeholder, WideModelSelector, ModelSelectionManager)
        except Exception as e:
            debug_print(f"Error importing/creating model selector in background: {e}")
            GLib.idle_add(self._on_models_loaded_error, str(e))

    def _create_model_selector_and_replace_placeholder(self, WideModelSelector, ModelSelectionManager):
        try:
            self._create_model_selector_if_needed(WideModelSelector, ModelSelectionManager)
            panel3_inner_vbox = self.panel3_placeholder.get_parent()
            if panel3_inner_vbox:
                panel3_inner_vbox.remove(self.panel3_placeholder)
                panel3_inner_vbox.append(self.model_selector)
            threading.Thread(
                target=self._load_models_data_in_thread,
                daemon=True,
                name="ModelDataLoader"
            ).start()
        except Exception as e:
            debug_print(f"Error creating model selector: {e}")
            GLib.idle_add(self._on_models_loaded_error, str(e))
        return False

    def _load_models_data_in_thread(self):
        try:
            # Cargar proveedores y modelos usando el manager del selector
            self.model_selector.load_providers_and_models()
            
            # Obtener el modelo por defecto
            try:
                import llm
                modelid = llm.get_default_model()
                debug_print(f"Debug: Modelo por defecto obtenido: {modelid}")
            except Exception as e:
                debug_print(f"Error obteniendo modelo por defecto: {e}")
                modelid = None
            
            GLib.idle_add(self._on_models_loaded_completed, modelid)
        except Exception as e:
            debug_print(f"Error loading model data in background: {e}")
            import traceback
            debug_print(traceback.format_exc())
            GLib.idle_add(self._on_models_loaded_error, str(e))

    def _on_models_loaded_completed(self, modelid=None):
        self._models_loaded = True
        self._ensure_loading_spinner_removed()
        if int(round(self.carousel.get_position())) == 2:
            self.update_navigation_buttons()
        
        if self.model_selector and modelid:
            debug_print(f"Debug: Intentando seleccionar modelo por defecto: {modelid}")
            # Verificar si el modelo requiere API key y si está configurada
            provider_key = None
            
            # Obtener el provider_key del modelo usando el manager
            if hasattr(self.model_selector, 'manager') and self.model_selector.manager:
                manager = self.model_selector.manager
                # Buscar el modelo en todos los proveedores
                for prov_key, models in manager.models_by_provider.items():
                    for model in models:
                        if getattr(model, 'model_id', None) == modelid:
                            provider_key = manager._provider_to_needs_key.get(prov_key)
                            debug_print(f"Debug: Encontrado modelo {modelid} en proveedor {prov_key}, needs_key: {provider_key}")
                            break
                    if provider_key is not None:
                        break
            
            # Si no se encontró, intentar obtener directamente del modelo
            if provider_key is None:
                try:
                    import llm
                    all_models = llm.get_models()
                    for m in all_models:
                        if getattr(m, 'model_id', None) == modelid:
                            provider_key = getattr(m, 'needs_key', None)
                            debug_print(f"Debug: Encontrado modelo {modelid} directamente, needs_key: {provider_key}")
                            break
                except Exception as e:
                    debug_print(f"Error buscando modelo en llm.get_models(): {e}")
                    provider_key = None
            
            # Consultar el estado de la API key si es necesario
            should_select = True
            if provider_key is not None and hasattr(self.model_manager, 'check_api_key_status'):
                key_status = self.model_manager.check_api_key_status(provider_key)
                needs_key = key_status.get('needs_key', False)
                has_key = key_status.get('has_key', False)
                debug_print(f"Debug: Estado de API key para {provider_key}: needs_key={needs_key}, has_key={has_key}")
                
                if needs_key and not has_key:
                    should_select = False
                    debug_print(f"Debug: No seleccionando modelo {modelid} porque falta API key para {provider_key}")
            
            # Seleccionar el modelo si es apropiado
            if should_select:
                debug_print(f"Debug: Seleccionando modelo por defecto: {modelid}")
                self.model_selector.pick_model(modelid)
        else:
            debug_print("Debug: No hay modelo por defecto para seleccionar")
            # Intentar seleccionar un modelo disponible
        
        return False

    def _on_models_loaded_error(self, error_message):
        debug_print(f"Background model loading failed: {error_message}")
        return False

    def _create_model_selector_if_needed(self, WideModelSelector, ModelSelectionManager):
        if not self._model_selector_created:
            self._model_selector_created = True
            self.model_manager = ModelSelectionManager(self.config_data, self.app.llm_client if hasattr(self.app, 'llm_client') else None)
            self.model_selector = WideModelSelector(manager=self.model_manager)
            self.model_selector.set_vexpand(True)
            self.model_selector.set_hexpand(True)
            self.model_selector.set_valign(Gtk.Align.FILL)
            self.model_selector.set_halign(Gtk.Align.FILL)
            self.model_selector.connect('model-selected', self._on_model_selected)
            self.model_selector.connect('api-key-status-changed', self._on_api_key_status_changed)
            # También conectar al stack interno para detectar cambios de proveedor
            if hasattr(self.model_selector, 'content_stack'):
                self.model_selector.content_stack.connect('notify::visible-child-name', self._on_provider_changed)

    def _on_tray_option_changed(self, checkbutton):
        if not checkbutton.get_active():
            return
        try:
            from .platform_utils import ensure_load_on_session_startup
            if self.tray_radio1.get_active():
                success = ensure_load_on_session_startup(True)
                if success:
                    debug_print("Autostart habilitado para inicio de sesión")
                    self.config_data['tray_startup'] = 'session'
                else:
                    debug_print("Error habilitando autostart")
            else:
                success = ensure_load_on_session_startup(False)
                if success:
                    debug_print("Autostart deshabilitado")
                    self.config_data['tray_startup'] = 'application'
                else:
                    debug_print("Error deshabilitando autostart")
        except Exception as e:
            debug_print(f"Error configurando autostart: {e}")

    def _initialize_tray_options(self):
        try:
            from .platform_utils import is_loading_on_session_startup
            autostart_enabled = is_loading_on_session_startup()
            if autostart_enabled:
                self.tray_radio1.set_active(True)
                self.config_data['tray_startup'] = 'session'
                debug_print("Autostart detectado como habilitado")
            else:
                self.tray_radio2.set_active(True)
                self.config_data['tray_startup'] = 'application'
                debug_print("Autostart detectado como deshabilitado")
        except Exception as e:
            debug_print(f"Error verificando estado de autostart: {e}")
            self.tray_radio2.set_active(True)
            self.config_data['tray_startup'] = 'application'

    def _on_provider_changed(self, stack, pspec):
        """Actualiza los botones cuando se cambia de proveedor."""
        current_provider = stack.get_visible_child_name()
        debug_print(f"Debug: Proveedor cambiado a: {current_provider}")
        if int(round(self.carousel.get_position())) == 2:
            # Usar un timeout pequeño para que el cambio se complete
            GLib.timeout_add(50, self.update_navigation_buttons)

    def _on_realize(self, widget):
        """Callback cuando la ventana se realiza - inicia la carga de modelos."""
        self.start_lazy_loading()

    def _on_window_show(self, window):
        """Configurar recursos de forma segura cuando se muestra la ventana."""
        if not hasattr(self, '_resources_configured'):
            try:
                style_manager.load_styles()
                resource_manager.setup_icon_theme()
                self._resources_configured = True
                debug_print("Welcome: Recursos configurados correctamente al mostrar ventana")
            except Exception as e:
                debug_print(f"Welcome: Error configurando recursos: {e}")

if __name__ == "__main__":
    import sys
    import signal
    app = Adw.Application(application_id="org.fuentelibre.GtkLLMChatWelcome", flags=Gio.ApplicationFlags.FLAGS_NONE)
    icon_theme = Gtk.IconTheme.get_for_display(Gdk.Display.get_default())
    project_root = os.path.dirname(os.path.abspath(__file__))
    icon_theme.add_search_path(project_root)
    settings = Gtk.Settings.get_default()
    if settings:
        settings.set_property("gtk-icon-theme-name", "Adwaita")
    def on_activate(app):
        def welcome_finished_callback(config):
            debug_print("Welcome flow finished. Config:", config)
            app.quit()
        win = WelcomeWindow(app, on_welcome_finished=welcome_finished_callback)
        win.present()
        signal.signal(signal.SIGINT, lambda s, f: app.quit())
    app.connect('activate', on_activate)
    app.run(sys.argv)


from markdown_it import MarkdownIt
import re
import gi
gi.require_version('Gtk', '4.0')
from gi.repository import Gtk, Pango  # noqa: E402


class MarkdownView(Gtk.TextView):
    def __init__(self):
        super().__init__()
        self.set_wrap_mode(Gtk.WrapMode.WORD_CHAR)
        self.set_editable(False)
        self.set_cursor_visible(False)
        self.buffer = self.get_buffer()
        self.md = MarkdownIt().enable('strikethrough')

        # Get the TextTagTable from the buffer
        self.tag_table = self.buffer.get_tag_table()  # Corrected line

        # Create TextTags and add them to the table
        self.bold_tag = Gtk.TextTag(name="bold")
        self.bold_tag.set_property("weight", Pango.Weight.BOLD)
        self.tag_table.add(self.bold_tag)

        self.italic_tag = Gtk.TextTag(name="italic")
        self.italic_tag.set_property("style", Pango.Style.ITALIC)
        self.tag_table.add(self.italic_tag)

        self.strike_tag = Gtk.TextTag(name="strike")
        self.strike_tag.set_property("strikethrough", True)
        self.tag_table.add(self.strike_tag)

        self.hr_tag = Gtk.TextTag(name="hr_line")
        self.hr_tag.set_property("foreground", "#666666")
        self.hr_tag.set_property("scale", 0.3)
        self.hr_tag.set_property("rise", -500)
        self.hr_tag.set_property("justification", Gtk.Justification.CENTER)
        self.tag_table.add(self.hr_tag)

        self.heading_tags = {
            '1': Gtk.TextTag(name="h1"),
            '2': Gtk.TextTag(name="h2"),
            '3': Gtk.TextTag(name="h3"),
            '4': Gtk.TextTag(name="h4"),
            '5': Gtk.TextTag(name="h5"),
        }
        self.heading_tags['1'].set_property("weight", Pango.Weight.BOLD)
        self.heading_tags['1'].set_property("size", 24 * Pango.SCALE)
        self.heading_tags['2'].set_property("weight", Pango.Weight.BOLD)
        self.heading_tags['2'].set_property("size", 20 * Pango.SCALE)
        self.heading_tags['3'].set_property("weight", Pango.Weight.BOLD)
        self.heading_tags['3'].set_property("size", 16 * Pango.SCALE)
        self.heading_tags['4'].set_property("weight", Pango.Weight.BOLD)
        self.heading_tags['4'].set_property("size", 12 * Pango.SCALE)
        self.heading_tags['5'].set_property("weight", Pango.Weight.BOLD)
        self.heading_tags['5'].set_property("size", 10 * Pango.SCALE)
        for tag in self.heading_tags.values():
            self.tag_table.add(tag)

        self.code_tag = Gtk.TextTag(name="code")
        self.code_tag.set_property("family", "monospace")
        self.code_tag.set_property("background", "gray")
        self.tag_table.add(self.code_tag)

        self.code_inline_tag = Gtk.TextTag(name="code_inline")
        self.code_inline_tag.set_property("family", "monospace")
        self.code_inline_tag.set_property("background", "#444444")
        self.tag_table.add(self.code_inline_tag)

        self.thinking_tag = Gtk.TextTag(name="thinking")
        self.thinking_tag.set_property("style", Pango.Style.ITALIC)
        self.thinking_tag.set_property("scale", 0.8)
        self.thinking_tag.set_property("left-margin", 20)
        self.thinking_tag.set_property("right-margin", 20)
        self.tag_table.add(self.thinking_tag)

        self.blockquote_tag = Gtk.TextTag(name="blockquote")
        self.blockquote_tag.set_property("left-margin", 30)
        self.blockquote_tag.set_property("style", Pango.Style.ITALIC)
        self.blockquote_tag.set_property("background", "gray")
        self.tag_table.add(self.blockquote_tag)

        self.list_tags = {
            1: Gtk.TextTag(name="list_1"),
            2: Gtk.TextTag(name="list_2"),
            3: Gtk.TextTag(name="list_3"),
        }
        self.list_tags[1].set_property("left-margin", 30)
        self.list_tags[2].set_property("left-margin", 50)
        self.list_tags[3].set_property("left-margin", 70)
        for tag in self.list_tags.values():
            self.tag_table.add(tag)

        self.in_list_item = False
        self.in_ordered_list = False
        self.current_tags = []
        self.list_level = 0

    def set_markdown(self, text):
        return self.render_markdown(text)

    def process_thinking_tags(self, text):
        """
        Procesa las etiquetas <think> o <thinking> en el texto.
        Devuelve una lista de fragmentos alternando texto normal y pensamiento.
        Cada fragmento es una tupla (texto, es_pensamiento).
        """
        fragments = []
        think_pattern = re.compile(r'<think>(.*?)</think>', re.DOTALL)
        thinking_pattern = re.compile(r'<thinking>(.*?)</thinking>', re.DOTALL)

        all_matches = []
        for pattern in [think_pattern, thinking_pattern]:
            for match in pattern.finditer(text):
                all_matches.append(
                    (match.start(), match.end(), match.group(1)))

        all_matches.sort(key=lambda x: x[0])

        last_end = 0
        for start, end, content in all_matches:
            if start > last_end:
                fragments.append((text[last_end:start], False))
            fragments.append((content, True))
            last_end = end

        if last_end < len(text):
            fragments.append((text[last_end:], False))

        return fragments

    def render_markdown(self, text):
        self.buffer.set_text("", 0)
        fragments = self.process_thinking_tags(text)

        for fragment_text, is_thinking in fragments:
            if is_thinking:
                self.insert_thinking(fragment_text)
            else:
                self.render_markdown_fragment(fragment_text)

    def render_markdown_fragment(self, text):
        tokens = self.md.parse(text)
        self.apply_pango_format(tokens)

    def apply_pango_format(self, tokens):
        for token in tokens:
            if token.type == 'strong_open':
                self.apply_tag(self.bold_tag)
            elif token.type == 'strong_close':
                self.remove_tag(self.bold_tag)
            elif token.type == 'em_open':
                self.apply_tag(self.italic_tag)
            elif token.type == 'em_close':
                self.remove_tag(self.italic_tag)
            elif token.type == 's_open':
                self.apply_tag(self.strike_tag)
            elif token.type == 's_close':
                self.remove_tag(self.strike_tag)

            elif token.type == 'text':
                self.insert_text(token.content)
            elif token.type == 'paragraph_open':
                pass
            elif token.type == 'paragraph_close':
                self.insert_text("\n\n")

            elif token.type == 'heading_open':
                level = token.tag[1]
                if level in self.heading_tags:
                    self.apply_tag(self.heading_tags[level])
            elif token.type == 'heading_close':
                level = token.tag[1]
                self.remove_tag(self.heading_tags[level])
                self.insert_text("\n\n")
            elif token.type == 'fence':
                self.apply_tag(self.code_tag)
                self.insert_text(token.content)
                self.remove_tag(self.code_tag)
                self.insert_text("\n")
            elif token.type == 'inline':
                for child in token.children:
                    if child.type == 'text':
                        self.insert_text(child.content)
                    elif child.type == 'em_open':
                        self.apply_tag(self.italic_tag)
                    elif child.type == 'em_close':
                        self.remove_tag(self.italic_tag)
                    elif child.type == 'strong_open':
                        self.apply_tag(self.bold_tag)
                    elif child.type == 'strong_close':
                        self.remove_tag(self.bold_tag)
                    elif child.type == 'code_inline':
                        self.apply_tag(self.code_inline_tag)
                        self.insert_text(child.content)
                        self.remove_tag(self.code_inline_tag)
                    # Manejar tachado en elementos inline
                    elif child.type == 's_open':
                        self.apply_tag(self.strike_tag)
                    elif child.type == 's_close':
                        self.remove_tag(self.strike_tag)
            elif token.type == 'blockquote_open':
                self.insert_text("\n")
                self.apply_tag(self.blockquote_tag)
            elif token.type == 'blockquote_close':
                self.remove_tag(self.blockquote_tag)
                self.insert_text("\n")
            elif token.type == 'bullet_list_open':
                self.list_level += 1
                self.apply_tag(self.list_tags[min(self.list_level, 3)])
            elif token.type == 'bullet_list_close':
                self.list_level -= 1
                current_level = min(self.list_level + 1, 3)
                self.remove_tag(self.list_tags[current_level])
            elif token.type == 'ordered_list_open':
                self.list_level += 1
                self.in_ordered_list = True
                self.apply_tag(self.list_tags[min(self.list_level, 3)])
            elif token.type == 'ordered_list_close':
                self.list_level -= 1
                self.in_ordered_list = False
                current_level = min(self.list_level + 1, 3)
                self.remove_tag(self.list_tags[current_level])
            elif token.type == 'list_item_open':
                self.in_list_item = True
                if self.in_ordered_list:
                    item_number = token.info
                    self.insert_text(f"{item_number}. ")
                else:
                    if self.list_level == 1:
                        self.insert_text("• ")
                    elif self.list_level == 2:
                        self.insert_text("◦ ")
                    else:
                        self.insert_text("▪ ")
            elif token.type == 'list_item_close':
                self.in_list_item = False
            elif token.type == 'hr':
                self.insert_text("\n")
                self.apply_tag(self.hr_tag)
                self.insert_text("-" * 35)
                self.remove_tag(self.hr_tag)
                self.insert_text("\n\n")
            elif token.type == 'html_block':
                pass
            elif token.type == 'code_block':
                self.insert_text("\n")
                self.insert_text(token.content)
                self.insert_text("\n")
            else:
                print("Unknown markdown token:", token.type, flush=True)


    def insert_text(self, text):
        buf = self.buffer
        buf.create_mark("insert_start", buf.get_end_iter(), left_gravity=True)
        buf.insert(buf.get_end_iter(), text, -1)
        start = buf.get_iter_at_mark(buf.get_mark("insert_start"))
        end = start.copy()
        end.forward_chars(len(text))
        for tag in self.current_tags:
            buf.apply_tag(tag, start, end)
        buf.delete_mark(buf.get_mark("insert_start"))

    def insert_thinking(self, text):
        buf = self.buffer
        buf.create_mark("think_start", buf.get_end_iter(), left_gravity=True)
        buf.insert(buf.get_end_iter(), text, -1)
        start = buf.get_iter_at_mark(buf.get_mark("think_start"))
        end = start.copy()
        end.forward_chars(len(text))
        buf.apply_tag(self.thinking_tag, start, end)
        buf.delete_mark(buf.get_mark("think_start"))
        buf.insert(buf.get_end_iter(), "\n", -1)

    def apply_tag(self, tag):
        if tag not in self.current_tags:
            self.current_tags.append(tag)
        start, end = self.buffer.get_bounds()
        if not start.equal(end):
            self.buffer.apply_tag(tag, end, self.buffer.get_end_iter())

    def remove_tag(self, tag):
        if tag in self.current_tags:
            self.current_tags.remove(tag)
        start, end = self.buffer.get_bounds()
        self.buffer.remove_tag(tag, end, self.buffer.get_end_iter())


if __name__ == "__main__":
    app = Gtk.Application(application_id='org.fuentelibre.MarkdownDemo')

    def on_activate(app):
        win = Gtk.ApplicationWindow(application=app)
        win.set_title("Markdown TextView")
        win.set_default_size(400, 300)

        markdown_text = """# Título 1\n## Título 2\n### Título 3\nEste es un
        **texto en negrita** y _cursiva_.
        \n```\n"
        Este es un bloque de código.\n"
        var x = 10;\n"
        ```\n"
        \nLista de ejemplo:\n"
        * Elemento 1\n  * Subelemento 1.1\n  * Subelemento 1.2\n* Elemento 2
        * Elemento 3\n"
        \nLista numerada:\n"
        1. Primer elemento\n"
        2. Segundo elemento\n"
           1. Subelemento 2.1\n"
        \nTexto con `código en línea` y emoji 😊\n"
        hola `amigo` 😊\n"""

        markdown_view = MarkdownView()
        scrolled_window = Gtk.ScrolledWindow()
        scrolled_window.set_child(markdown_view)
        win.set_child(scrolled_window)

        markdown_view.render_markdown(markdown_text)
        win.present()

    app.connect('activate', on_activate)
    app.run()
